library(extraDistr)
library(Matrix)
#Performance measure--Evaluate L1 loss between the private data and the observed data
compute_L1_loss=function(gdat, dat){
  l=abs(gdat[,1]-dat[1])
  for (i in 2:length(dat)){
    l=cbind(l, abs(gdat[,i]-dat[i]))
  }
  return(l)
}

#To evaluate the proportion negatives
prop_negative=function(d){
  count=0
  for (i in 1:nrow(d)){
    if (any(d[i, ]<0)){
      count=count+1
    }
  }
  return(count/nrow(d))
}


###Optimal mechanism functions
#Returns a truncated geometric mechanism probability matrix at 0 and n
truncated_geometric=function(n, eps){ 
  a=exp(-eps)
  x=matrix(0, n+1, n+1)
  x[,1]=(a^(0:n))/(1+a)
  x[, n+1]=(a^(n:0))/(1+a)
  x[1, (2:n)]=(a^(1:(n-1)))*(1-a)/(1+a)
  x[n+1, (2:n)]=(a^((n-1):1))*(1-a)/(1+a)
  for (i in 2:n){
    for (r in 2:n){
      x[i,r]=(a^abs(i-r))*(1-a)/(1+a)
    }
  }
  return(x)
}

#Evaluate the posterior probability matrix h
posterior=function(x, p){#p is the prior vector
  n=length(p)-1
  p_c=(p%*%t(rep(1, (n+1))))*x
  denomenator=apply(p_c, 2, sum)
  pos=p_c/(rep(1, (n+1))%*%t(denomenator))
  return(pos)
}

#Evaluate L1 loss matrix for L.
l1_loss=function(n){
  l=matrix(0, n+1, n+1)
  for (i in 1:(n+1)){
    for (j in 1:(n+1)){
      l[i,j]=abs(i-j)
    }
  }
  return(l)
}


#Find median of a vector
find_median=function(vector){
  return(which(vector>0.5)[1])
}


#Evaluate the optimal remap matrix
optimal_remap_new=function(n, eps, p){
  a=exp(-eps)
  x=truncated_geometric(n, eps)
  pos=posterior(x, p)
  csum=apply(pos, 2, cumsum)
  opt_j=apply(csum, 2, find_median)-1
  y=matrix(0, n+1, n+1)
  r=1:(n+1)
  for (indx in 1:(n+1)){
    y[r[indx], opt_j[indx]+1]=1
  }
  return(y)
}

#Evaluate the perturbation probability matrix for the optimal procedure
perturbation_matrix=function(n, eps, p){
  x=truncated_geometric(n, eps)
  y=optimal_remap_new(n, eps, p)
  return(x%*%y)
}

#Generate size samples of private data using the optimal mechanism
generate_optimal_data=function(n, eps, p, size, obs){#obs is the actual observed data value
  opt_mat=perturbation_matrix(n, eps, p)
  pro=opt_mat[obs+1,]
  res=sample(0:n, prob = pro, replace = T, size = size)
  return(res)
}



####Hypothesis testing using optimal DP####
#Evalute empirical type I errors using naive Chi-squared statistic for the optimal mechanism
#p is the prior, alpha is the targeted level of significance
  typeI_multiple_opt_naive=function(opt_mat, eps, size, n, p0, alpha, p){
    x_value=rmultinom(n=1, size=n, prob=p0)
    K=length(p0)
    x_s=c()
    for (i in 1:K){
      pro=opt_mat[x_value[i]+1,]
      x_c=sample(0:n, prob = pro, replace = T, size = 1)
      x_s=c(x_s, x_c)
    }
    
    for (i in 2:size){
      x_value=rmultinom(n=1, size=n, prob=p0)
      x_c=c()
      for (i in 1:K){
        pro=opt_mat[x_value[i]+1,]
        c=sample(0:n, prob = pro, replace = T, size = 1)
        x_c=c(x_c, c)
      }
      x_s=rbind(x_s, x_c)
    }
    t=c()
    for (i in 1:size){
      t_c=sum(((x_s[i,]-n*p0)^2)/(n*p0))
      t=c(t, t_c)
    }
    thres_u=qchisq(1-alpha, df=K-1)  
    typeI=100*(length(which(t>=thres_u)))/size
    return(typeI)
  }
  
  
####Opt mechanism empirical type I using covariance with bias correction####
##Theorem 1
eva_normalised_prob=function(x){#here x is a vector
  x/sum(x)
}

#Return the expected bias [d*1, d*2,..,d*K]
bias=function(n, eps){
  x=truncated_geometric(n, eps)
  bias=matrix(rep(0:n,n+1), nrow=n+1, byrow = T)
  bias=bias-c(0:n)%*%t(rep(1, n+1))
  bias=apply(bias*x, 1, sum)
  pos=apply(x, 2, eva_normalised_prob)
  b=apply(pos*(bias%*%t(rep(1, n+1))), 2, sum)
  return(bias)
}

#Return the diagonal entries of Sigma
variance=function(n, eps){
  x=truncated_geometric(n, eps)
  bias=matrix(rep(0:n,n+1), nrow=n+1, byrow = T)
  bias=bias-c(0:n)%*%t(rep(1, n+1))
  vari=apply(x*(bias^2), 1, sum)-(apply(bias*x, 1, sum))^2
  return(mean(vari))
}

#p is the prior, alpha is the targeted level of significance
typeI_multiple_opt_with_bias_reduction=function(opt_mat,eps, size, n, p0, alpha, p){
  b=bias(n, eps)
  v=variance(n, eps)
  vari=(n*p0*(1-p0)+v)/n/p0
  K=length(p0)
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]= vari[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  x_value=rmultinom(n=1, size=n, prob=p0)
  x_s=c()
  for (i in 1:K){
    pro=opt_mat[x_value[i]+1,]
    x_c=sample(0:n, prob = pro, replace = T, size = 1)
    x_s=c(x_s, x_c-b[x_value[i]+1])
  }
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p0)
    x_c=c()
    for (i in 1:K){
      pro=opt_mat[x_value[i]+1,]
      c=sample(0:n, prob = pro, replace = T, size = 1)
      x_c=c(x_c, c-b[x_value[i]+1])
    }
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p0)^2)/(n*p0))
    t=c(t, t_c)
  }
  typeI=100*(length(which(t>=threshold)))/size
  return(typeI)
}


##To evaluate empirical powers

#Opt mechanism without bias correction
#p0, p1 are null hypothesis and alternative hypothesis respectively
#alpha is the targeted level of significance
power_multiple_opt_without_bias_reduction=function(opt_mat, eps, size, n, p0, alpha, p1){
  v=variance(n, eps)
  vari=(n*p0*(1-p0)+v)/n/p0
  K=length(p0)
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]= vari[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:500){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  x_value=rmultinom(n=1, size=n, prob=p1)
  x_s=c()
  for (i in 1:K){
    pro=opt_mat[x_value[i]+1,]
    x_c=sample(0:n, prob = pro, replace = T, size = 1)
    x_s=c(x_s, x_c)
  }
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p1)
    x_c=c()
    for (i in 1:K){
      pro=opt_mat[x_value[i]+1,]
      c=sample(0:n, prob = pro, replace = T, size = 1)
      x_c=c(x_c, c)
    }
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p0)^2)/(n*p0))
    t=c(t, t_c)
  }
  pow=100*(length(which(t>=threshold)))/size
  return(pow)
}



#To evaluate the empirical power: Opt mechanism with bias correction
#p0, p1 are null hypothesis and alternative hypothesis respectively
#alpha is the targeted level of significance
power_multiple_opt_with_bias_reduction=function(opt_mat, eps, size, n, p0, alpha, p1){
  b=bias(n, eps)
  v=variance(n, eps)
  vari=(n*p0*(1-p0)+v)/n/p0
  K=length(p0)
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]= vari[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  x_value=rmultinom(n=1, size=n, prob=p1)
  x_s=c()
  for (i in 1:K){
    pro=opt_mat[x_value[i]+1,]
    x_c=sample(0:n, prob = pro, replace = T, size = 1)
    x_s=c(x_s, x_c-b[x_value[i]+1])
  }
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p1)
    x_c=c()
    for (i in 1:K){
      pro=opt_mat[x_value[i]+1,]
      c=sample(0:n, prob = pro, replace = T, size = 1)
      x_c=c(x_c, c-b[x_value[i]+1])
    }
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p0)^2)/(n*p0))
    t=c(t, t_c)
  }
  pow=100*(length(which(t>=threshold)))/size
  return(pow)
}


###Inter-table merging
#Evaluate empirical type I error rates for Opt mechanism for inter-table (2 tables here) merged private data
#n1, n2 are the total count of table 1 and table 2.
#p1, p2 are the prior probabilities for table 1 and 2.
#p0 is the null hypothesized values
#alpha is the targeted level of significance
typeI_merge_multiple_opt_with_bias_reduction=function(eps, size, n1, n2, p0, alpha, p1, p2){
  opt_mat1=perturbation_matrix(n1, eps, p1)
  opt_mat2=perturbation_matrix(n2, eps, p2)
  b1=bias(n1, eps)
  b2=bias(n2, eps)
  v1=variance(n1, eps)
  v2=variance(n2, eps)
  vari=((n1+n2)*p0*(1-p0)+v1+v2)/(n1+n2)/p0
  K=length(p0)
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]= vari[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  #Generate the samples for the first set of data
  x_value1=rmultinom(n=1, size=n1, prob=p0)
  x_s1=c()
  for (i in 1:K){
    pro1=opt_mat1[x_value1[i]+1,]
    x_c1=sample(0:n1, prob = pro1, replace = T, size = 1)
    x_s1=c(x_s1, x_c1-b1[x_value1[i]+1])
  }
  
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p0)
    x_c1=c()
    for (i in 1:K){
      pro1=opt_mat1[x_value1[i]+1,]
      c1=sample(0:n1, prob = pro1, replace = T, size = 1)
      x_c1=c(x_c1, c1-b1[x_value1[i]+1])
    }
    x_s1=rbind(x_s1, x_c1)
  }
  #Generate the samples for the second set of data
  x_value2=rmultinom(n=1, size=n2, prob=p0)
  x_s2=c()
  for (i in 1:K){
    pro2=opt_mat2[x_value2[i]+1,]
    x_c2=sample(0:n2, prob = pro2, replace = T, size = 1)
    x_s2=c(x_s2, x_c2-b2[x_value2[i]+1])
  }
  
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p0)
    x_c2=c()
    for (i in 1:K){
      pro2=opt_mat2[x_value2[i]+1,]
      c2=sample(0:n2, prob = pro2, replace = T, size = 1)
      x_c2=c(x_c2, c2-b2[x_value2[i]+1])
    }
    x_s2=rbind(x_s2, x_c2)
  }
  
  x_s=x_s1+x_s2
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-(n1+n2)*p0)^2)/((n1+n2)*p0))
    t=c(t, t_c)
  }
  typeI=100*(length(which(t>=threshold)))/size
  return(typeI)
}

#Evaluate empirical power for Opt mechanism for inter-table (2 tables here) merged private data
#n1, n2 are the total count of table 1 and table 2.
#opt_mat1, opt_mat2 are the optimal perturbation probability matrices for table 1 and 2 respectively.
#p0 is the null hypothesized value
#p1 is the alternative hypothesized value
#alpha is the targeted level of significance
power_merge_multiple_opt_with_bias_reduction=function(eps, size, n1, n2, p0, alpha, p1, opt_mat1, opt_mat2){
  b1=bias(n1, eps)
  b2=bias(n2, eps)
  v1=variance(n1, eps)
  v2=variance(n2, eps)
  vari=((n1+n2)*p0*(1-p0)+v1+v2)/(n1+n2)/p0
  K=length(p0)
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]= vari[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  
  #Generate the samples for the first set of data
  x_value1=rmultinom(n=1, size=n1, prob=p1)
  x_s1=c()
  for (i in 1:K){
    pro1=opt_mat1[x_value1[i]+1,]
    x_c1=sample(0:n1, prob = pro1, replace = T, size = 1)
    x_s1=c(x_s1, x_c1-b1[x_value1[i]+1])
  }
  
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p1)
    x_c1=c()
    for (i in 1:K){
      pro1=opt_mat1[x_value1[i]+1,]
      c1=sample(0:n1, prob = pro1, replace = T, size = 1)
      x_c1=c(x_c1, c1-b1[x_value1[i]+1])
    }
    x_s1=rbind(x_s1, x_c1)
  }
  #Generate the samples for the second set of data
  x_value2=rmultinom(n=1, size=n2, prob=p1)
  x_s2=c()
  for (i in 1:K){
    pro2=opt_mat2[x_value2[i]+1,]
    x_c2=sample(0:n2, prob = pro2, replace = T, size = 1)
    x_s2=c(x_s2, x_c2-b2[x_value2[i]+1])
  }
  
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p1)
    x_c2=c()
    for (i in 1:K){
      pro2=opt_mat2[x_value2[i]+1,]
      c2=sample(0:n2, prob = pro2, replace = T, size = 1)
      x_c2=c(x_c2, c2-b2[x_value2[i]+1])
    }
    x_s2=rbind(x_s2, x_c2)
  }
  
  x_s=x_s1+x_s2
  
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-(n1+n2)*p0)^2)/((n1+n2)*p0))
    t=c(t, t_c)
  }
  pow=100*(length(which(t>=threshold)))/size
  return(pow)
}



##Intra-table merging
#To evaluate the empirical type I errors
#opt_mat is the optimal perturbation probability matrix
#n is the total count of table 
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#merge_idx is a list of groups of indices of the table entries to merge together
typeI_opt_intercellmerge_manycell=function(opt_mat, eps, size, n, p0, alpha,  merge_idx){
  K=length(p0)
  b=bias(n, eps)
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  v=count*variance(n, eps)
  vari=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=length(p_merge), nrow=length(p_merge))
  for (i in 1:(length(p_merge))){
    for (j in 1:(length(p_merge))){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=vari[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  
  x_value=rmultinom(n=1, size=n, prob=p0)
  x_s=c()
  for (i in 1:K){
    pro=opt_mat[x_value[i]+1,]
    x_c=sample(0:n, prob = pro, replace = T, size = 1)
    x_s=c(x_s, x_c-b[x_value[i]+1])
  }
  x_merge=c()
  for (i in merge_idx){
    x_merge=c(x_merge, sum(x_s[i]))
  }
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p0)
    x_c=c()
    for (i in 1:K){
      pro=opt_mat[x_value[i]+1,]
      c=sample(0:n, prob = pro, replace = T, size = 1)
      x_c=c(x_c, c-b[x_value[i]+1])
    }
    x_m=c()
    for (i in merge_idx){
      x_m=c(x_m, sum(x_c[i]))
    }
    x_merge=rbind(x_merge, x_m)
  }
  
  t=c()
  for (i in 1:size){
    t_c=sum(((x_merge[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  typeI=100*(length(which(t>=threshold)))/size
  return(typeI)
}


#To evaluate average p-values
#opt_mat is the optimal perturbation probability matrix
#n is the total count of table 
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#merge_idx is a list of groups of indices of the table entries to merge together
p_value_opt_intercellmerge_manycell=function(opt_mat,eps, size, n, p0, alpha, p, x_value, merge_idx){
  K=length(p0)
  b=bias(n, eps)
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  v=count*variance(n, eps)
  variance=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=length(p_merge), nrow=length(p_merge))
  for (i in 1:(length(p_merge))){
    for (j in 1:(length(p_merge))){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  x_s=c()
  for (i in 1:K){
    pro=opt_mat[x_value[i]+1,]
    x_c=sample(0:n, prob = pro, replace = T, size = 1)
    x_s=c(x_s, x_c-b[x_value[i]+1])
  }
  x_merge=c()
  for (i in merge_idx){
    x_merge=c(x_merge, sum(x_s[i]))
  }
  
  for (i in 2:size){
    x_c=c()
    for (i in 1:K){
      pro=opt_mat[x_value[i]+1, ]
      c=sample(0:n, prob = pro, replace = T, size = 1)
      x_c=c(x_c, c-b[x_value[i]+1])
    }
    x_m=c()
    for (i in merge_idx){
      x_m=c(x_m, sum(x_c[i]))
    }
    x_merge=rbind(x_merge, x_m)
  }
  
  t=c()
  for (i in 1:size){
    t_c=sum(((x_merge[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  stats_derived=sort(stats_derived)
  p_v=c()
  for (i in 1:length(t)){
    p_v=c(p_v, which(stats_derived>t[i])[1]/1000)
  }
  if (is.na(mean(p_v, na.rm=T)==T)){
    return(0)
  }
  else{
    return(1-mean(p_v, na.rm=T))
  }
}




#Evaluate empirical power
#opt_mat is the optimal perturbation probability matrix
#n is the total count of table 
#p0 is the null hypothesized value
#p1 is the alternative hypothesized value
#alpha is the targeted level of significance
#merge_idx is a list of groups of indices of the table entries to merge together
pow_opt_intercellmerge_manycell=function(opt_mat,eps, size, n, p0, alpha,  p1, merge_idx){
  K=length(p0)
  b=bias(n, eps)
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  v=count*variance(n, eps)
  variance=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=length(p_merge), nrow=length(p_merge))
  for (i in 1:(length(p_merge))){
    for (j in 1:(length(p_merge))){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  
  x_value=rmultinom(n=1, size=n, prob=p1)
  x_s=c()
  for (i in 1:K){
    pro=opt_mat[x_value[i]+1,]
    x_c=sample(0:n, prob = pro, replace = T, size = 1)
    x_s=c(x_s, x_c-b[x_value[i]+1])
  }
  x_merge=c()
  for (i in merge_idx){
    x_merge=c(x_merge, sum(x_s[i]))
  }
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p1)
    x_c=c()
    for (i in 1:K){
      pro=opt_mat[x_value[i]+1,]
      c=sample(0:n, prob = pro, replace = T, size = 1)
      x_c=c(x_c, c-b[x_value[i]+1])
    }
    x_m=c()
    for (i in merge_idx){
      x_m=c(x_m, sum(x_c[i]))
    }
    x_merge=rbind(x_merge, x_m)
  }
  
  t=c()
  for (i in 1:size){
    t_c=sum(((x_merge[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  typeI=100*(length(which(t>=threshold)))/size
  return(typeI)
}


##Specifically for the real data application part
##Combining three test statistics##

#To evaluate empirical type I errors
#n1, n2, n3 are the total counts for tables 1, 2 and 3 respectively
#p01, p02, p03 are the null hypothesized values for table 1, 2 and 3 respectively
#p1, p2, p3 prior probabilities for table 1, 2 and 3 respectively
#alpha is the targeted level of significance
typeI_combine_opt=function(eps, size, n1, n2, n3, p01, p02, p03, alpha, p1, p2, p3){
  opt_mat1=perturbation_matrix(n1, eps, p1)
  b1=bias(n1, eps)
  v1=variance(n1, eps)
  vari1=(n1*p01*(1-p01)+v1)/n1/p01
  K1=length(p01)
  cov1=matrix(0, ncol=K1, nrow=K1)
  for (i in 1:K1){
    for (j in 1:K1){
      if (i!=j){
        cov1[i,j]=-sqrt(p01[i]*p01[j])
      }
      if (i==j){
        cov1[i,i]= vari1[i]
      }
    }
  }
  
  opt_mat2=perturbation_matrix(n2, eps, p2)
  b2=bias(n2, eps)
  v2=variance(n2, eps)
  vari2=(n2*p02*(1-p02)+v2)/n2/p02
  K2=length(p02)
  cov2=matrix(0, ncol=K2, nrow=K2)
  for (i in 1:K2){
    for (j in 1:K2){
      if (i!=j){
        cov2[i,j]=-sqrt(p02[i]*p02[j])
      }
      if (i==j){
        cov2[i,i]= vari2[i]
      }
    }
  }
  
  opt_mat3=perturbation_matrix(n3, eps, p3)
  b3=bias(n3, eps)
  v3=variance(n3, eps)
  vari3=(n3*p03*(1-p03)+v3)/n3/p03
  K3=length(p03)
  cov3=matrix(0, ncol=K3, nrow=K3)
  for (i in 1:K3){
    for (j in 1:K3){
      if (i!=j){
        cov3[i,j]=-sqrt(p03[i]*p03[j])
      }
      if (i==j){
        cov3[i,i]= vari3[i]
      }
    }
  }
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=K1+K2+K3, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  x_value1=rmultinom(n=1, size=n1, prob=p01)
  x_s1=c()
  for (i in 1:K1){
    pro1=opt_mat1[x_value1[i]+1,]
    x_c1=sample(0:n1, prob = pro1, replace = T, size = 1)
    x_s1=c(x_s1, x_c1-b1[x_value1[i]+1])
  }
  
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p01)
    x_c1=c()
    for (i in 1:K1){
      pro1=opt_mat1[x_value1[i]+1,]
      c1=sample(0:n1, prob = pro1, replace = T, size = 1)
      x_c1=c(x_c1, c1-b1[x_value1[i]+1])
    }
    x_s1=rbind(x_s1, x_c1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_s1[i,]-n1*p01)^2)/(n1*p01))
    t1=c(t1, t_c1)
  }
  
  x_value2=rmultinom(n=1, size=n2, prob=p02)
  x_s2=c()
  for (i in 1:K2){
    pro2=opt_mat2[x_value2[i]+1,]
    x_c2=sample(0:n2, prob = pro2, replace = T, size = 1)
    x_s2=c(x_s2, x_c2-b2[x_value2[i]+1])
  }
  
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p02)
    x_c2=c()
    for (i in 1:K2){
      pro2=opt_mat2[x_value2[i]+1,]
      c2=sample(0:n2, prob = pro2, replace = T, size = 1)
      x_c2=c(x_c2, c2-b2[x_value2[i]+1])
    }
    x_s2=rbind(x_s2, x_c2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_s2[i,]-n2*p02)^2)/(n2*p02))
    t2=c(t2, t_c2)
  }
  
  x_value3=rmultinom(n=1, size=n3, prob=p03)
  x_s3=c()
  for (i in 1:K3){
    pro3=opt_mat3[x_value3[i]+1,]
    x_c3=sample(0:n3, prob = pro3, replace = T, size = 1)
    x_s3=c(x_s3, x_c3-b3[x_value3[i]+1])
  }
  
  for (i in 2:size){
    x_value3=rmultinom(n=1, size=n3, prob=p03)
    x_c3=c()
    for (i in 1:K3){
      pro3=opt_mat3[x_value3[i]+1,]
      c3=sample(0:n3, prob = pro3, replace = T, size = 1)
      x_c3=c(x_c3, c3-b3[x_value3[i]+1])
    }
    x_s3=rbind(x_s3, x_c3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_s3[i,]-n3*p03)^2)/(n3*p03))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  typeI=100*(length(which(t>=threshold)))/size
  return(typeI)
}


#To evaluate average p-value for the combined test statistic.
#n1, n2, n3 are the total counts for tables 1, 2 and 3 respectively.
#p01, p02, p03 are the null hypothesized values for tables 1, 2 and 3 respectively.
#x_value1,x_value2,x_value3 are observed values for tables 1, 2 and 3 respectively.
#alpha is the targeted level of significance.
#opt_mat1, opt_mat2, opt_mat3 are optimal perturbation matrix for tables 1, 2 and 3 respectively.
p_val_combine_opt=function(eps, size, n1, n2, n3, p01, p02, p03, x_value1,x_value2,x_value3,alpha, opt_mat1, opt_mat2, opt_mat3){
  b1=bias(n1, eps)
  v1=variance(n1, eps)
  vari1=(n1*p01*(1-p01)+v1)/n1/p01
  K1=length(p01)
  cov1=matrix(0, ncol=K1, nrow=K1)
  for (i in 1:K1){
    for (j in 1:K1){
      if (i!=j){
        cov1[i,j]=-sqrt(p01[i]*p01[j])
      }
      if (i==j){
        cov1[i,i]= vari1[i]
      }
    }
  }
  

  b2=bias(n2, eps)
  v2=variance(n2, eps)
  vari2=(n2*p02*(1-p02)+v2)/n2/p02
  K2=length(p02)
  cov2=matrix(0, ncol=K2, nrow=K2)
  for (i in 1:K2){
    for (j in 1:K2){
      if (i!=j){
        cov2[i,j]=-sqrt(p02[i]*p02[j])
      }
      if (i==j){
        cov2[i,i]= vari2[i]
      }
    }
  }
  

  b3=bias(n3, eps)
  v3=variance(n3, eps)
  vari3=(n3*p03*(1-p03)+v3)/n3/p03
  K3=length(p03)
  cov3=matrix(0, ncol=K3, nrow=K3)
  for (i in 1:K3){
    for (j in 1:K3){
      if (i!=j){
        cov3[i,j]=-sqrt(p03[i]*p03[j])
      }
      if (i==j){
        cov3[i,i]= vari3[i]
      }
    }
  }
  
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=K1+K2+K3, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }

  
  x_s1=c()
  for (i in 1:K1){
    pro1=opt_mat1[x_value1[i]+1,]
    x_c1=sample(0:n1, prob = pro1, replace = T, size = 1)
    x_s1=c(x_s1, x_c1-b1[x_value1[i]+1])
  }
  
  for (i in 2:size){
    x_c1=c()
    for (i in 1:K1){
      pro1=opt_mat1[x_value1[i]+1,]
      c1=sample(0:n1, prob = pro1, replace = T, size = 1)
      x_c1=c(x_c1, c1-b1[x_value1[i]+1])
    }
    x_s1=rbind(x_s1, x_c1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_s1[i,]-n1*p01)^2)/(n1*p01))
    t1=c(t1, t_c1)
  }
  

  x_s2=c()
  for (i in 1:K2){
    pro2=opt_mat2[x_value2[i]+1,]
    x_c2=sample(0:n2, prob = pro2, replace = T, size = 1)
    x_s2=c(x_s2, x_c2-b2[x_value2[i]+1])
  }
  
  for (i in 2:size){
 
    x_c2=c()
    for (i in 1:K2){
      pro2=opt_mat2[x_value2[i]+1,]
      c2=sample(0:n2, prob = pro2, replace = T, size = 1)
      x_c2=c(x_c2, c2-b2[x_value2[i]+1])
    }
    x_s2=rbind(x_s2, x_c2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_s2[i,]-n2*p02)^2)/(n2*p02))
    t2=c(t2, t_c2)
  }
  
 
  x_s3=c()
  for (i in 1:K3){
    pro3=opt_mat3[x_value3[i]+1,]
    x_c3=sample(0:n3, prob = pro3, replace = T, size = 1)
    x_s3=c(x_s3, x_c3-b3[x_value3[i]+1])
  }
  
  for (i in 2:size){
    x_c3=c()
    for (i in 1:K3){
      pro3=opt_mat3[x_value3[i]+1,]
      c3=sample(0:n3, prob = pro3, replace = T, size = 1)
      x_c3=c(x_c3, c3-b3[x_value3[i]+1])
    }
    x_s3=rbind(x_s3, x_c3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_s3[i,]-n3*p03)^2)/(n3*p03))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  stats_derived=sort(stats_derived)
  p_v=c()
  for (i in 1:length(t)){
    p_v=c(p_v, which(stats_derived>t[i])[1]/1000)
  }
  return(1-mean(p_v, na.rm=T))
}


#Evaluate the empirical powers for combined statistic
#n1, n2, n3 are the total counts for tables 1, 2 and 3 respectively.
#p01, p02, p03 are the null hypothesized values for tables 1, 2 and 3 respectively.
#p11, p12, p13 are the alternative hypothesized values for tables 1, 2 and 3 respectively.
#alpha is the targeted level of significance.
#opt_mat1, opt_mat2, opt_mat3 are optimal perturbation matrix for tables 1, 2 and 3 respectively.
pow_combine_opt=function(eps, size, n1, n2,n3, p01, p02, p03, p11, p12, p13, alpha, opt_mat1, opt_mat2, opt_mat3){
  b1=bias(n1, eps)
  v1=variance(n1, eps)
  vari1=(n1*p01*(1-p01)+v1)/n1/p01
  K1=length(p01)
  cov1=matrix(0, ncol=K1, nrow=K1)
  for (i in 1:K1){
    for (j in 1:K1){
      if (i!=j){
        cov1[i,j]=-sqrt(p01[i]*p01[j])
      }
      if (i==j){
        cov1[i,i]= vari1[i]
      }
    }
  }
  
  b2=bias(n2, eps)
  v2=variance(n2, eps)
  vari2=(n2*p02*(1-p02)+v2)/n2/p02
  K2=length(p02)
  cov2=matrix(0, ncol=K2, nrow=K2)
  for (i in 1:K2){
    for (j in 1:K2){
      if (i!=j){
        cov2[i,j]=-sqrt(p02[i]*p02[j])
      }
      if (i==j){
        cov2[i,i]= vari2[i]
      }
    }
  }
  
  b3=bias(n3, eps)
  v3=variance(n3, eps)
  vari3=(n3*p03*(1-p03)+v3)/n3/p03
  K3=length(p03)
  cov3=matrix(0, ncol=K3, nrow=K3)
  for (i in 1:K3){
    for (j in 1:K3){
      if (i!=j){
        cov3[i,j]=-sqrt(p03[i]*p03[j])
      }
      if (i==j){
        cov3[i,i]= vari3[i]
      }
    }
  }
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=K1+K2+K3, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  x_value1=rmultinom(n=1, size=n1, prob=p11)
  x_s1=c()
  for (i in 1:K1){
    pro1=opt_mat1[x_value1[i]+1,]
    x_c1=sample(0:n1, prob = pro1, replace = T, size = 1)
    x_s1=c(x_s1, x_c1-b1[x_value1[i]+1])
  }
  
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p11)
    x_c1=c()
    for (i in 1:K1){
      pro1=opt_mat1[x_value1[i]+1,]
      c1=sample(0:n1, prob = pro1, replace = T, size = 1)
      x_c1=c(x_c1, c1-b1[x_value1[i]+1])
    }
    x_s1=rbind(x_s1, x_c1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_s1[i,]-n1*p01)^2)/(n1*p01))
    t1=c(t1, t_c1)
  }
  
  x_value2=rmultinom(n=1, size=n2, prob=p12)
  x_s2=c()
  for (i in 1:K2){
    pro2=opt_mat2[x_value2[i]+1,]
    x_c2=sample(0:n2, prob = pro2, replace = T, size = 1)
    x_s2=c(x_s2, x_c2-b2[x_value2[i]+1])
  }
  
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p12)
    x_c2=c()
    for (i in 1:K2){
      pro2=opt_mat2[x_value2[i]+1,]
      c2=sample(0:n2, prob = pro2, replace = T, size = 1)
      x_c2=c(x_c2, c2-b2[x_value2[i]+1])
    }
    x_s2=rbind(x_s2, x_c2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_s2[i,]-n2*p02)^2)/(n2*p02))
    t2=c(t2, t_c2)
  }
  
  x_value3=rmultinom(n=1, size=n3, prob=p13)
  x_s3=c()
  for (i in 1:K3){
    pro3=opt_mat3[x_value3[i]+1,]
    x_c3=sample(0:n3, prob = pro3, replace = T, size = 1)
    x_s3=c(x_s3, x_c3-b3[x_value3[i]+1])
  }
  
  for (i in 2:size){
    x_value3=rmultinom(n=1, size=n3, prob=p13)
    x_c3=c()
    for (i in 1:K3){
      pro3=opt_mat3[x_value3[i]+1,]
      c3=sample(0:n3, prob = pro3, replace = T, size = 1)
      x_c3=c(x_c3, c3-b3[x_value3[i]+1])
    }
    x_s3=rbind(x_s3, x_c3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_s3[i,]-n3*p03)^2)/(n3*p03))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  pow=100*(length(which(t>=threshold)))/size
  return(pow)
}








###Specifically for Income variable

#To evaluate average p-values after intra-table merging
#n1, n2, n3 are the total counts for tables 1, 2 and 3 respectively.
#p0 is the common null hypothesized value.
#p1, p2, p3 are the prior probabilities for tables 1, 2 and 3 respectively.
#x_value1, x_value2, x_value3 are the actual observations for tables 1, 2 and 3 respectively.
#alpha is the targeted level of significance.
#merge_idx is a list of groups of indices of the table entries to merge together
p_value_opt_intercellmerge_manycell_income=function(eps, size, n1, n2, n3, p0, alpha, p1, p2, p3, x_value1, x_value2, x_value3, merge_idx){
  K=length(p0)
  opt_mat1=perturbation_matrix(n1, eps, p1)
  b1=bias(n1, eps)
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  v1=count*variance(n1, eps)
  
  variance1=(n1*p_merge*(1-p_merge)+v1)/n1/p_merge
  cov1=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        cov1[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        cov1[i,i]=variance1[i]
      }
    }
  }
  
  v2=count*variance(n2, eps)
  variance2=(n2*p_merge*(1-p_merge)+v2)/n2/p_merge
  cov2=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        cov2[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        cov2[i,i]=variance2[i]
      }
    }
  }
  
  v3=count*variance(n3, eps)
  variance3=(n3*p_merge*(1-p_merge)+v3)/n3/p_merge
  cov3=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        cov3[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        cov3[i,i]=variance3[i]
      }
    }
  }
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:10000){
    stat=sum(lam*rchisq(n=3*length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  
  
  #Generate statistics
  opt_mat1=perturbation_matrix(n1, eps, p1)
  b1=bias(n1, eps)
  x_s1=c()
  for (i in 1:K){
    pro1=opt_mat1[x_value1[i]+1,]
    x_c1=sample(0:n1, prob = pro1, replace = T, size = 1)
    x_s1=c(x_s1, x_c1-b1[x_value1[i]+1])
  }
  x_merge1=c()
  for (i in merge_idx){
    x_merge1=c(x_merge1, sum(x_s1[i]))
  }
  
  for (i in 2:size){
    x_c1=c()
    for (i in 1:K){
      pro1=opt_mat1[x_value1[i]+1,]
      c1=sample(0:n1, prob = pro1, replace = T, size = 1)
      x_c1=c(x_c1, c1-b1[x_value1[i]+1])
    }
    x_m1=c()
    for (i in merge_idx){
      x_m1=c(x_m1, sum(x_c1[i]))
    }
    x_merge1=rbind(x_merge1, x_m1)
  }
  
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_merge1[i,]-n1*p_merge)^2)/(n1*p_merge))
    t1=c(t1, t_c1)
  }
  
  #Second
  opt_mat2=perturbation_matrix(n2, eps, p2)
  b2=bias(n2, eps)
  x_s2=c()
  for (i in 1:K){
    pro2=opt_mat2[x_value2[i]+1,]
    x_c2=sample(0:n2, prob = pro2, replace = T, size = 1)
    x_s2=c(x_s2, x_c2-b2[x_value2[i]+1])
  }
  x_merge2=c()
  for (i in merge_idx){
    x_merge2=c(x_merge2, sum(x_s2[i]))
  }
  
  for (i in 2:size){
    x_c2=c()
    for (i in 1:K){
      pro2=opt_mat2[x_value2[i]+1,]
      c2=sample(0:n2, prob = pro2, replace = T, size = 1)
      x_c2=c(x_c2, c2-b2[x_value2[i]+1])
    }
    x_m2=c()
    for (i in merge_idx){
      x_m2=c(x_m2, sum(x_c2[i]))
    }
    x_merge2=rbind(x_merge2, x_m2)
  }
  
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_merge2[i,]-n2*p_merge)^2)/(n2*p_merge))
    t2=c(t2, t_c2)
  }
  
  
  #Third
  opt_mat3=perturbation_matrix(n3, eps, p3)
  b3=bias(n3, eps)
  x_s3=c()
  for (i in 1:K){
    pro3=opt_mat3[x_value3[i]+1,]
    x_c3=sample(0:n3, prob = pro3, replace = T, size = 1)
    x_s3=c(x_s3, x_c3-b3[x_value3[i]+1])
  }
  x_merge3=c()
  for (i in merge_idx){
    x_merge3=c(x_merge3, sum(x_s3[i]))
  }
  
  for (i in 2:size){
    x_c3=c()
    for (i in 1:K){
      pro3=opt_mat3[x_value3[i]+1,]
      c3=sample(0:n3, prob = pro3, replace = T, size = 1)
      x_c3=c(x_c3, c3-b3[x_value3[i]+1])
    }
    x_m3=c()
    for (i in merge_idx){
      x_m3=c(x_m3, sum(x_c3[i]))
    }
    x_merge3=rbind(x_merge3, x_m3)
  }
  
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_merge3[i,]-n3*p_merge)^2)/(n3*p_merge))
    t3=c(t3, t_c3)
  }
  
  t=t1+t2+t2
  stats_derived=sort(stats_derived)
  p_v=c()
  for (i in 1:length(t)){
    p_v=c(p_v, which(stats_derived>t[i])[1]/10000)
  }
  if (is.na(mean(p_v, na.rm=T)==T)){
    return(0)
  }
  else{
    return(1-mean(p_v, na.rm=T))
  }
}











#######Laplace mechanism functions ########

#Generate private data from Laplace mechanism
generate_laplace_nom=function(eps, x, size, n){
  L=rlaplace(n=size, mu=0, sigma=1/eps)
  dat=round(x+L)
  return(dat)
}

#Generate private data from Laplace mechanism with truncation at 0
generate_laplace_nom_trunc=function(eps, x, size, n){
  L=rlaplace(n=size, mu=0, sigma=1/eps)
  dat=round(x+L)
  dat[dat<0]=0
  return(dat)
}

####Goodness of fit test####
##Many cell testing ##

##To evaluate empirical type I errors using naive Chi-squared statistics
#For Lap
#n is the total count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
typeI_multiple_lap_naive=function(eps, size, n, p0, alpha){
  x_value=rmultinom(n=1, size=n, prob=p0)
  K=length(p0)
  x_s=c()
  for (i in 1:K){
    x_c=generate_laplace_nom(eps, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p0)
    x_c=c()
    for (i in 1:K){
      c=generate_laplace_nom(eps, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p0)^2)/(n*p0))
    t=c(t, t_c)
  }
  thres_u=qchisq(1-alpha, df=K-1)  
  typeI=100*(length(which(t>=thres_u)))/size
  return(typeI)
}


#For TLap
#n is the total count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
typeI_multiple_lap_naive_trunc=function(eps, size, n, p0, alpha){
  x_value=rmultinom(n=1, size=n, prob=p0)
  K=length(p0)
  x_s=c()
  for (i in 1:K){
    x_c=generate_laplace_nom_trunc(eps, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p0)
    x_c=c()
    for (i in 1:K){
      c=generate_laplace_nom_trunc(eps, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p0)^2)/(n*p0))
    t=c(t, t_c)
  }
  thres_u=qchisq(1-alpha, df=K-1)  
  typeI=100*(length(which(t>=thres_u)))/size
  return(typeI)
}







##To evaluate empirical type I errors using proposed procedure in Theorem 2##

#For Lap
#n is the total count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
type1_lap=function(eps, n, p0, alpha, size){
  K=length(p0)
  variance=(n*p0*(1-p0)+2/(eps^2))/n/p0
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  x_value=rmultinom(n=1, size=n, prob=p0)
  x_s=c()
  for (i in 1:K){
    x_c=generate_laplace_nom(eps, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p0)
    x_c=c()
    for (i in 1:K){
      c=generate_laplace_nom(eps, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p0)^2)/(n*p0))
    t=c(t, t_c)
  }
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}

#For TLap
#n is the total count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
type1_lap_trunc=function(eps, n, p0, alpha, size){
  K=length(p0)
  variance=(n*p0*(1-p0)+2/eps^2)/n/p0
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  x_value=rmultinom(n=1, size=n, prob=p0)
  x_s=c()
  for (i in 1:K){
    x_c=generate_laplace_nom_trunc(eps, x=x_value[i], size=1, n)
    x_s=c(x_s, max(0, x_c))
  }
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p0)
    x_c=c()
    for (i in 1:K){
      c=generate_laplace_nom_trunc(eps, x=x_value[i], size=1, n)
      x_c=c(x_c, max(0, c))
    }
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p0)^2)/(n*p0))
    t=c(t, t_c)
  }
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}


#Evaluate empirical powers using private data generated from Lap/TLap
#For Lap
#n is the total count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#p1 is the alternative hypothesized value
power_lap_multiple=function(n, p0, alpha, size, p1, eps){
  K=length(p0)
  variance=(n*p0*(1-p0)+2/eps^2)/n/p0
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  x_value=rmultinom(n=1, size=n, prob=p1)
  x_s=c()
  for (i in 1:K){
    x_c=generate_laplace_nom(eps, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p1)
    x_c=c()
    for (i in 1:K){
      c=generate_laplace_nom(eps, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p0)^2)/(n*p0))
    t=c(t, t_c)
  }
  return(100*(length(which(t>=threshold)))/size) 
}

#For TLap
#n is the total count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#p1 is the alternative hypothesized value
power_lap_multiple_trunc=function(n, p0, alpha, size, p1, eps){
  K=length(p0)
  variance=(n*p0*(1-p0)+2/eps^2)/n/p0
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  x_value=rmultinom(n=1, size=n, prob=p1)
  x_s=c()
  for (i in 1:K){
    x_c=generate_laplace_nom_trunc(eps, x=x_value[i], size=1, n)
    x_s=c(x_s, max(0, x_c))
  }
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p1)
    x_c=c()
    for (i in 1:K){
      c=generate_laplace_nom_trunc(eps, x=x_value[i], size=1, n)
      x_c=c(x_c, max(0, c))
    }
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p0)^2)/(n*p0))
    t=c(t, t_c)
  }
  return(100*(length(which(t>=threshold)))/size) 
}



###Inter-table merging for merging two tables###

#For Lap
#n1, n2 are the total counts for tables 1 and 2 respectively.
#p0 is the common null hypothesized value
#alpha is the targeted level of significance
type1_merge_lap=function(eps, n1, n2, p0, alpha, size){
  K=length(p0)
  variance=((n1+n2)*p0*(1-p0)+4/eps^2)/(n1+n2)/p0
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  x_value1=rmultinom(n=1, size=n1, prob=p0)
  x_s1=c()
  for (i in 1:K){
    x_c1=generate_laplace_nom(eps, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p0)
    x_c1=c()
    for (i in 1:K){
      c1=generate_laplace_nom(eps, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_s1=rbind(x_s1, x_c1)
  }
  x_value2=rmultinom(n=1, size=n2, prob=p0)
  x_s2=c()
  for (i in 1:K){
    x_c2=generate_laplace_nom(eps, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p0)
    x_c2=c()
    for (i in 1:K){
      c2=generate_laplace_nom(eps, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  
  x_s=x_s1+x_s2
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-(n1+n2)*p0)^2)/((n1+n2)*p0))
    t=c(t, t_c)
  }
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}


#For TLap
#n1, n2 are the total counts for tables 1 and 2 respectively.
#p0 is the common null hypothesized value
#alpha is the targeted level of significance
type1_merge_lap_trunc=function(eps, n1, n2, p0, alpha, size){
  K=length(p0)
  variance=((n1+n2)*p0*(1-p0)+4/eps^2)/(n1+n2)/p0
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  x_value1=rmultinom(n=1, size=n1, prob=p0)
  x_s1=c()
  for (i in 1:K){
    x_c1=generate_laplace_nom_trunc(eps, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p0)
    x_c1=c()
    for (i in 1:K){
      c1=generate_laplace_nom_trunc(eps, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_s1=rbind(x_s1, x_c1)
  }
  x_value2=rmultinom(n=1, size=n2, prob=p0)
  x_s2=c()
  for (i in 1:K){
    x_c2=generate_laplace_nom_trunc(eps, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p0)
    x_c2=c()
    for (i in 1:K){
      c2=generate_laplace_nom_trunc(eps, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  
  x_s=x_s1+x_s2
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-(n1+n2)*p0)^2)/((n1+n2)*p0))
    t=c(t, t_c)
  }
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}


##Evaluate empirical powers in the inter-table merging case
#For Lap
#n1, n2 are the total counts for tables 1 and 2 respectively.
#p0 is the common null hypothesized value
#alpha is the targeted level of significance
#p1 is the alternative hypothesized value
pow_merge_lap=function(eps, n1, n2, p0, alpha, size, p1){
  K=length(p0)
  variance=((n1+n2)*p0*(1-p0)+4/eps^2)/(n1+n2)/p0
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  x_value1=rmultinom(n=1, size=n1, prob=p1)
  x_s1=c()
  for (i in 1:K){
    x_c1=generate_laplace_nom(eps, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p1)
    x_c1=c()
    for (i in 1:K){
      c1=generate_laplace_nom(eps, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_s1=rbind(x_s1, x_c1)
  }
  
  x_value2=rmultinom(n=1, size=n2, prob=p1)
  x_s2=c()
  for (i in 1:K){
    x_c2=generate_laplace_nom(eps, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p1)
    x_c2=c()
    for (i in 1:K){
      c2=generate_laplace_nom(eps, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  
  x_s=x_s1+x_s2
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-(n1+n2)*p0)^2)/((n1+n2)*p0))
    t=c(t, t_c)
  }
  pow=100*length(which(t>=threshold))/size
  return(pow)
}



#For TLap
#n1, n2 are the total counts for tables 1 and 2 respectively.
#p0 is the common null hypothesized value
#alpha is the targeted level of significance
#p1 is the alternative hypothesized value
pow_merge_lap_trunc=function(eps, n1, n2, p0, alpha, size, p1){
  K=length(p0)
  variance=((n1+n2)*p0*(1-p0)+4/eps^2)/(n1+n2)/p0
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  x_value1=rmultinom(n=1, size=n1, prob=p1)
  x_s1=c()
  for (i in 1:K){
    x_c1=generate_laplace_nom_trunc(eps, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p1)
    x_c1=c()
    for (i in 1:K){
      c1=generate_laplace_nom_trunc(eps, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_s1=rbind(x_s1, x_c1)
  }
  
  x_value2=rmultinom(n=1, size=n2, prob=p1)
  x_s2=c()
  for (i in 1:K){
    x_c2=generate_laplace_nom_trunc(eps, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p1)
    x_c2=c()
    for (i in 1:K){
      c2=generate_laplace_nom_trunc(eps, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  
  x_s=x_s1+x_s2
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-(n1+n2)*p0)^2)/((n1+n2)*p0))
    t=c(t, t_c)
  }
  pow=100*length(which(t>=threshold))/size
  return(pow)
}


###Intra-table merging case###
#Evaluate empirical type I errors 

#For the Lap
#n is the total table count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#merge_idx is a list of groups of indices of the table entries to merge
typeI_multiple_lap_intercell_merge=function(eps, size, n, p0, alpha, merge_idx){
  K=length(p0)
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  #p_merge=c(sum(p0[1:C]), p0[(C+1):K])
  v=count*2/eps^2
  variance=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  #Generate statistics
  x_value=rmultinom(n=1, size=n, prob=p0)
  x_s=c()
  for (i in 1:K){
    x_c=generate_laplace_nom(eps, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  x_merge=c()
  for (i in merge_idx){
    x_merge=c(x_merge, sum(x_s[i]))
  }
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p0)
    x_c=c()
    for (i in 1:K){
      c=generate_laplace_nom(eps, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_m=c()
    for (i in merge_idx){
      x_m=c(x_m, sum(x_c[i]))
    }
    x_merge=rbind(x_merge, x_m)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_merge[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}


#For the TLap
#n is the total table count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#merge_idx is a list of groups of indices of the table entries to merge
typeI_multiple_lap_intercell_merge_trunc=function(eps, size, n, p0, alpha, merge_idx){
  K=length(p0)
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  #p_merge=c(sum(p0[1:C]), p0[(C+1):K])
  v=count*2/eps^2
  variance=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  #Generate statistics
  x_value=rmultinom(n=1, size=n, prob=p0)
  x_s=c()
  for (i in 1:K){
    x_c=generate_laplace_nom_trunc(eps, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  x_merge=c()
  for (i in merge_idx){
    x_merge=c(x_merge, sum(x_s[i]))
  }
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p0)
    x_c=c()
    for (i in 1:K){
      c=generate_laplace_nom_trunc(eps, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_m=c()
    for (i in merge_idx){
      x_m=c(x_m, sum(x_c[i]))
    }
    x_merge=rbind(x_merge, x_m)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_merge[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}



#Evaluate average p-values
#For the Lap
#n is the total table count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#x_value is the actual observed values
#merge_idx is a list of groups of indices of the table entries to merge
p_value_multiple_lap_intercell_merge=function(eps, size, n, p0, alpha, x_value, merge_idx){
  K=length(p0)
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  v=count*2/eps^2
  variance=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  x_s=c()
  for (i in 1:K){
    x_c=generate_laplace_nom(eps, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  x_merge=c()
  for (i in merge_idx){
    x_merge=c(x_merge, sum(x_s[i]))
  }
  
  for (i in 2:size){
    x_c=c()
    for (i in 1:K){
      c=generate_laplace_nom(eps, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_m=c()
    for (i in merge_idx){
      x_m=c(x_m, sum(x_c[i]))
    }
    x_merge=rbind(x_merge, x_m)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_merge[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  stats_derived=sort(stats_derived)
  p_v=c()
  for (i in 1:length(t)){
    p_v=c(p_v, which(stats_derived>t[i])[1]/1000)
  }
  if (is.na(mean(p_v, na.rm=T)==T)){
    return(0)
  }
  else{
    return(1-mean(p_v, na.rm=T))
  }
}

#For the TLap
#n is the total table count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#x_value is the actual observed values
#merge_idx is a list of groups of indices of the table entries to merge
p_value_multiple_lap_intercell_merge_trunc=function(eps, size, n, p0, alpha, x_value, merge_idx){
  K=length(p0)
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  v=count*2/eps^2
  variance=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  x_s=c()
  for (i in 1:K){
    x_c=generate_laplace_nom_trunc(eps, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  x_merge=c()
  for (i in merge_idx){
    x_merge=c(x_merge, sum(x_s[i]))
  }
  
  for (i in 2:size){
    x_c=c()
    for (i in 1:K){
      c=generate_laplace_nom_trunc(eps, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_m=c()
    for (i in merge_idx){
      x_m=c(x_m, sum(x_c[i]))
    }
    x_merge=rbind(x_merge, x_m)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_merge[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  stats_derived=sort(stats_derived)
  p_v=c()
  for (i in 1:length(t)){
    p_v=c(p_v, which(stats_derived>t[i])[1]/1000)
  }
  if (is.na(mean(p_v, na.rm=T)==T)){
    return(0)
  }
  else{
    return(1-mean(p_v, na.rm=T))
  }
}


##To evaluate the empirical powers
#For the Lap
#n is the total table count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#p1 is the alternative hypothesized value
#merge_idx is a list of groups of indices of the table entries to merge
pow_multiple_lap_intercell_merge=function(eps, size, n, p0, alpha, p1, merge_idx){#p is the prior
  K=length(p0)
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  #p_merge=c(sum(p0[1:C]), p0[(C+1):K])
  v=count*2/eps^2
  variance=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  #Generate statistics
  x_value=rmultinom(n=1, size=n, prob=p1)
  x_s=c()
  for (i in 1:K){
    x_c=generate_laplace_nom(eps, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  x_merge=c()
  for (i in merge_idx){
    x_merge=c(x_merge, sum(x_s[i]))
  }
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p1)
    x_c=c()
    for (i in 1:K){
      c=generate_laplace_nom(eps, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_m=c()
    for (i in merge_idx){
      x_m=c(x_m, sum(x_c[i]))
    }
    x_merge=rbind(x_merge, x_m)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_merge[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  pow=100*length(which(t>=threshold))/size
  return(pow)
}

#For the TLap
#n is the total table count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#p1 is the alternative hypothesized value
#merge_idx is a list of groups of indices of the table entries to merge
pow_multiple_lap_intercell_merge_trunc=function(eps, size, n, p0, alpha, p1, merge_idx){#p is the prior
  K=length(p0)
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  v=count*2/eps^2
  variance=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  #Generate statistics
  x_value=rmultinom(n=1, size=n, prob=p1)
  x_s=c()
  for (i in 1:K){
    x_c=generate_laplace_nom_trunc(eps, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  x_merge=c()
  for (i in merge_idx){
    x_merge=c(x_merge, sum(x_s[i]))
  }
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p1)
    x_c=c()
    for (i in 1:K){
      c=generate_laplace_nom_trunc(eps, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_m=c()
    for (i in merge_idx){
      x_m=c(x_m, sum(x_c[i]))
    }
    x_merge=rbind(x_merge, x_m)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_merge[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}



###For real data application only (Lap) ####
##Combining multiple test statistics (3)##
#Evaluate empirical type I errors for the combined statistics
#For Lap
#n1, n2, n3 are the total counts for tables 1, 2 and 3, respectively
#p01,p02,p03 are the null hypothesized values for tables 1, 2 and 3, respectively
#alpha is the targeted level of significance
typeI_combine_lap=function(eps, n1, n2, n3, p01,p02,p03, alpha, size){
  K1=length(p01)
  v1=(n1*p01*(1-p01)+2/eps^2)/n1/p01
  cov1=matrix(0, ncol=K1, nrow=K1)
  for (i in 1:K1){
    for (j in 1:K1){
      if (i!=j){
        cov1[i,j]=-sqrt(p01[i]*p01[j])
      }
      if (i==j){
        cov1[i,i]=v1[i]
      }
    }
  }
  
  K2=length(p02)
  v2=(n2*p02*(1-p02)+2/eps^2)/n2/p02
  cov2=matrix(0, ncol=K2, nrow=K2)
  for (i in 1:K2){
    for (j in 1:K2){
      if (i!=j){
        cov2[i,j]=-sqrt(p02[i]*p02[j])
      }
      if (i==j){
        cov2[i,i]=v2[i]
      }
    }
  }
  
  K3=length(p03)
  v3=(n3*p03*(1-p03)+2/eps^2)/n3/p03
  cov3=matrix(0, ncol=K3, nrow=K3)
  for (i in 1:K3){
    for (j in 1:K3){
      if (i!=j){
        cov3[i,j]=-sqrt(p03[i]*p03[j])
      }
      if (i==j){
        cov3[i,i]=v3[i]
      }
    }
  }

  e1=eigen(cov1)
  lam1=e1$values
  stats_derived1=c()
  for (j in 1:1000){
    stat=sum(lam1*rchisq(n=K1, df=1, ncp = 0))
    stats_derived1=c(stats_derived1, stat)
  }
  e2=eigen(cov2)
  lam2=e2$values
  stats_derived2=c()
  for (j in 1:1000){
    stat=sum(lam2*rchisq(n=K2, df=1, ncp = 0))
    stats_derived2=c(stats_derived2, stat)
  }
  e3=eigen(cov3)
  lam3=e3$values
  stats_derived3=c()
  for (j in 1:1000){
    stat=sum(lam3*rchisq(n=K3, df=1, ncp = 0))
    stats_derived3=c(stats_derived3, stat)
  }
  stats_derived=stats_derived1+stats_derived2+stats_derived3
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  
  x_value1=rmultinom(n=1, size=n1, prob=p01)
  x_s1=c()
  for (i in 1:K1){
    x_c1=generate_laplace_nom(eps, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p01)
    x_c1=c()
    for (i in 1:K1){
      c1=generate_laplace_nom(eps, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_s1=rbind(x_s1, x_c1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_s1[i,]-n1*p01)^2)/(n1*p01))
    t1=c(t1, t_c1)
  }
  
  
  x_value2=rmultinom(n=1, size=n2, prob=p02)
  x_s2=c()
  for (i in 1:K2){
    x_c2=generate_laplace_nom(eps, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p02)
    x_c2=c()
    for (i in 1:K2){
      c2=generate_laplace_nom(eps, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_s2[i,]-n2*p02)^2)/(n2*p02))
    t2=c(t2, t_c2)
  }
  
  x_value3=rmultinom(n=1, size=n3, prob=p03)
  x_s3=c()
  for (i in 1:K3){
    x_c3=generate_laplace_nom(eps, x=x_value3[i], size=1, n3)
    x_s3=c(x_s3, x_c3)
  }
  for (i in 2:size){
    x_value3=rmultinom(n=1, size=n3, prob=p03)
    x_c3=c()
    for (i in 1:K3){
      c3=generate_laplace_nom(eps, x=x_value3[i], size=1, n3)
      x_c3=c(x_c3, c3)
    }
    x_s3=rbind(x_s3, x_c3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_s3[i,]-n3*p03)^2)/(n3*p03))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}


#For TLap
#n1, n2, n3 are the total counts for tables 1, 2 and 3, respectively
#p01,p02,p03 are the null hypothesized values for tables 1, 2 and 3, respectively
#alpha is the targeted level of significance

typeI_combine_lap_trunc=function(eps, n1, n2, n3, p01, p02, p03, alpha, size){
  K1=length(p01)
  v1=(n1*p01*(1-p01)+2/eps^2)/n1/p01
  cov1=matrix(0, ncol=K1, nrow=K1)
  for (i in 1:K1){
    for (j in 1:K1){
      if (i!=j){
        cov1[i,j]=-sqrt(p01[i]*p01[j])
      }
      if (i==j){
        cov1[i,i]=v1[i]
      }
    }
  }
  
  K2=length(p02)
  v2=(n2*p02*(1-p02)+2/eps^2)/n2/p02
  cov2=matrix(0, ncol=K2, nrow=K2)
  for (i in 1:K2){
    for (j in 1:K2){
      if (i!=j){
        cov2[i,j]=-sqrt(p02[i]*p02[j])
      }
      if (i==j){
        cov2[i,i]=v2[i]
      }
    }
  }
  
  K3=length(p03)
  v3=(n3*p03*(1-p03)+2/eps^2)/n3/p03
  cov3=matrix(0, ncol=K3, nrow=K3)
  for (i in 1:K3){
    for (j in 1:K3){
      if (i!=j){
        cov3[i,j]=-sqrt(p03[i]*p03[j])
      }
      if (i==j){
        cov3[i,i]=v3[i]
      }
    }
  }

  e1=eigen(cov1)
  lam1=e1$values
  stats_derived1=c()
  for (j in 1:1000){
    stat=sum(lam1*rchisq(n=K1, df=1, ncp = 0))
    stats_derived1=c(stats_derived1, stat)
  }
  e2=eigen(cov2)
  lam2=e2$values
  stats_derived2=c()
  for (j in 1:1000){
    stat=sum(lam2*rchisq(n=K2, df=1, ncp = 0))
    stats_derived2=c(stats_derived2, stat)
  }
  e3=eigen(cov3)
  lam3=e3$values
  stats_derived3=c()
  for (j in 1:1000){
    stat=sum(lam3*rchisq(n=K3, df=1, ncp = 0))
    stats_derived3=c(stats_derived3, stat)
  }
  stats_derived=stats_derived1+stats_derived2+stats_derived3
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  
  x_value1=rmultinom(n=1, size=n1, prob=p01)
  x_s1=c()
  for (i in 1:K1){
    x_c1=generate_laplace_nom_trunc(eps, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p01)
    x_c1=c()
    for (i in 1:K1){
      c1=generate_laplace_nom_trunc(eps, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, max(c1, 0))
    }
    x_s1=rbind(x_s1, x_c1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_s1[i,]-n1*p01)^2)/(n1*p01))
    t1=c(t1, t_c1)
  }
  
  
  x_value2=rmultinom(n=1, size=n2, prob=p02)
  x_s2=c()
  for (i in 1:K2){
    x_c2=generate_laplace_nom_trunc(eps, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, max(x_c2, 0))
  }
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p02)
    x_c2=c()
    for (i in 1:K2){
      c2=generate_laplace_nom_trunc(eps, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2,  max(c2, 0))
    }
    x_s2=rbind(x_s2, x_c2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_s2[i,]-n2*p02)^2)/(n2*p02))
    t2=c(t2, t_c2)
  }
  
  x_value3=rmultinom(n=1, size=n3, prob=p03)
  x_s3=c()
  for (i in 1:K3){
    x_c3=generate_laplace_nom_trunc(eps, x=x_value3[i], size=1, n3)
    x_s3=c(x_s3, x_c3)
  }
  for (i in 2:size){
    x_value3=rmultinom(n=1, size=n3, prob=p03)
    x_c3=c()
    for (i in 1:K3){
      c3=generate_laplace_nom_trunc(eps, x=x_value3[i], size=1, n3)
      x_c3=c(x_c3,  max(c3, 0))
    }
    x_s3=rbind(x_s3, x_c3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_s3[i,]-n3*p03)^2)/(n3*p03))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}

###To evaluate average p-values
#For Lap
#n1, n2, n3 are the total counts for tables 1, 2 and 3, respectively
#p01,p02,p03 are the null hypothesized values for tables 1, 2 and 3, respectively
#x_value1, x_value2, x_value3 are the observed values for tables 1, 2 and 3, respectively
#alpha is the targeted level of significance
p_value_combine_lap=function(eps, n1, n2, n3, p01,p02,p03, x_value1, x_value2, x_value3, alpha, size){
  K1=length(p01)
  v1=(n1*p01*(1-p01)+2/eps^2)/n1/p01
  cov1=matrix(0, ncol=K1, nrow=K1)
  for (i in 1:K1){
    for (j in 1:K1){
      if (i!=j){
        cov1[i,j]=-sqrt(p01[i]*p01[j])
      }
      if (i==j){
        cov1[i,i]=v1[i]
      }
    }
  }
  
  K2=length(p02)
  v2=(n2*p02*(1-p02)+2/eps^2)/n2/p02
  cov2=matrix(0, ncol=K2, nrow=K2)
  for (i in 1:K2){
    for (j in 1:K2){
      if (i!=j){
        cov2[i,j]=-sqrt(p02[i]*p02[j])
      }
      if (i==j){
        cov2[i,i]=v2[i]
      }
    }
  }
  
  K3=length(p03)
  v3=(n3*p03*(1-p03)+2/eps^2)/n3/p03
  cov3=matrix(0, ncol=K3, nrow=K3)
  for (i in 1:K3){
    for (j in 1:K3){
      if (i!=j){
        cov3[i,j]=-sqrt(p03[i]*p03[j])
      }
      if (i==j){
        cov3[i,i]=v3[i]
      }
    }
  }
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=K1+K2+K3, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  
  x_s1=c()
  for (i in 1:K1){
    x_c1=generate_laplace_nom(eps, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  for (i in 2:size){

    x_c1=c()
    for (i in 1:K1){
      c1=generate_laplace_nom(eps, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_s1=rbind(x_s1, x_c1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_s1[i,]-n1*p01)^2)/(n1*p01))
    t1=c(t1, t_c1)
  }
  
  

  x_s2=c()
  for (i in 1:K2){
    x_c2=generate_laplace_nom(eps, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  for (i in 2:size){

    x_c2=c()
    for (i in 1:K2){
      c2=generate_laplace_nom(eps, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_s2[i,]-n2*p02)^2)/(n2*p02))
    t2=c(t2, t_c2)
  }
  

  x_s3=c()
  for (i in 1:K3){
    x_c3=generate_laplace_nom(eps, x=x_value3[i], size=1, n3)
    x_s3=c(x_s3, x_c3)
  }
  for (i in 2:size){
    
    x_c3=c()
    for (i in 1:K3){
      c3=generate_laplace_nom(eps, x=x_value3[i], size=1, n3)
      x_c3=c(x_c3, c3)
    }
    x_s3=rbind(x_s3, x_c3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_s3[i,]-n3*p03)^2)/(n3*p03))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  stats_derived=sort(stats_derived)
  p_v=c()
  for (i in 1:length(t)){
    p_v=c(p_v, which(stats_derived>t[i])[1]/1000)
  }
  return(1-mean(p_v, na.rm=T))
}

#For TLap
#n1, n2, n3 are the total counts for tables 1, 2 and 3, respectively
#p01,p02,p03 are the null hypothesized values for tables 1, 2 and 3, respectively
#x_value1, x_value2, x_value3 are the observed values for tables 1, 2 and 3, respectively
#alpha is the targeted level of significance
p_value_combine_lap_trunc=function(eps, n1, n2, n3, p01,p02,p03, x_value1, x_value2, x_value3, alpha, size){
  K1=length(p01)
  v1=(n1*p01*(1-p01)+2/eps^2)/n1/p01
  cov1=matrix(0, ncol=K1, nrow=K1)
  for (i in 1:K1){
    for (j in 1:K1){
      if (i!=j){
        cov1[i,j]=-sqrt(p01[i]*p01[j])
      }
      if (i==j){
        cov1[i,i]=v1[i]
      }
    }
  }
  
  K2=length(p02)
  v2=(n2*p02*(1-p02)+2/eps^2)/n2/p02
  cov2=matrix(0, ncol=K2, nrow=K2)
  for (i in 1:K2){
    for (j in 1:K2){
      if (i!=j){
        cov2[i,j]=-sqrt(p02[i]*p02[j])
      }
      if (i==j){
        cov2[i,i]=v2[i]
      }
    }
  }
  
  K3=length(p03)
  v3=(n3*p03*(1-p03)+2/eps^2)/n3/p03
  cov3=matrix(0, ncol=K3, nrow=K3)
  for (i in 1:K3){
    for (j in 1:K3){
      if (i!=j){
        cov3[i,j]=-sqrt(p03[i]*p03[j])
      }
      if (i==j){
        cov3[i,i]=v3[i]
      }
    }
  }
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=K1+K2+K3, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  
  x_s1=c()
  for (i in 1:K1){
    x_c1=generate_laplace_nom_trunc(eps, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  for (i in 2:size){
    x_c1=c()
    for (i in 1:K1){
      c1=generate_laplace_nom_trunc(eps, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_s1=rbind(x_s1, x_c1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_s1[i,]-n1*p01)^2)/(n1*p01))
    t1=c(t1, t_c1)
  }
  
  

  x_s2=c()
  for (i in 1:K2){
    x_c2=generate_laplace_nom_trunc(eps, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  for (i in 2:size){
    x_c2=c()
    for (i in 1:K2){
      c2=generate_laplace_nom_trunc(eps, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_s2[i,]-n2*p02)^2)/(n2*p02))
    t2=c(t2, t_c2)
  }
  
  x_s3=c()
  for (i in 1:K3){
    x_c3=generate_laplace_nom_trunc(eps, x=x_value3[i], size=1, n3)
    x_s3=c(x_s3, x_c3)
  }
  for (i in 2:size){
    x_c3=c()
    for (i in 1:K3){
      c3=generate_laplace_nom_trunc(eps, x=x_value3[i], size=1, n3)
      x_c3=c(x_c3, c3)
    }
    x_s3=rbind(x_s3, x_c3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_s3[i,]-n3*p03)^2)/(n3*p03))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  stats_derived=sort(stats_derived)
  p_v=c()
  for (i in 1:length(t)){
    p_v=c(p_v, which(stats_derived>t[i])[1]/1000)
  }
  return(1-mean(p_v, na.rm=T))
}


#To evaluate empirical powers
#For Lap
#n1, n2, n3 are the total counts for tables 1, 2 and 3, respectively
#p01,p02,p03 are the null hypothesized values for tables 1, 2 and 3, respectively
#p11,p12,p13 are the alternative hypothesized values for tables 1, 2 and 3, respectively
#x_value1, x_value2, x_value3 are the observed values for tables 1, 2 and 3, respectively
#alpha is the targeted level of significance
power_combine_lap=function(eps, n1, n2, n3, p01,p02,p03,p11,p12,p13, alpha, size){
  K1=length(p01)
  v1=(n1*p01*(1-p01)+2/eps^2)/n1/p01
  cov1=matrix(0, ncol=K1, nrow=K1)
  for (i in 1:K1){
    for (j in 1:K1){
      if (i!=j){
        cov1[i,j]=-sqrt(p01[i]*p01[j])
      }
      if (i==j){
        cov1[i,i]=v1[i]
      }
    }
  }
  
  K2=length(p02)
  v2=(n2*p02*(1-p02)+2/eps^2)/n2/p02
  cov2=matrix(0, ncol=K2, nrow=K2)
  for (i in 1:K2){
    for (j in 1:K2){
      if (i!=j){
        cov2[i,j]=-sqrt(p02[i]*p02[j])
      }
      if (i==j){
        cov2[i,i]=v2[i]
      }
    }
  }
  
  K3=length(p03)
  v3=(n3*p03*(1-p03)+2/eps^2)/n3/p03
  cov3=matrix(0, ncol=K3, nrow=K3)
  for (i in 1:K3){
    for (j in 1:K3){
      if (i!=j){
        cov3[i,j]=-sqrt(p03[i]*p03[j])
      }
      if (i==j){
        cov3[i,i]=v3[i]
      }
    }
  }
  
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=K1+K2+K3, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  x_value1=rmultinom(n=1, size=n1, prob=p11)
  x_s1=c()
  for (i in 1:K1){
    x_c1=generate_laplace_nom(eps, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p11)
    x_c1=c()
    for (i in 1:K1){
      c1=generate_laplace_nom(eps, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_s1=rbind(x_s1, x_c1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_s1[i,]-n1*p01)^2)/(n1*p01))
    t1=c(t1, t_c1)
  }
  
  
  x_value2=rmultinom(n=1, size=n2, prob=p12)
  x_s2=c()
  for (i in 1:K2){
    x_c2=generate_laplace_nom(eps, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p12)
    x_c2=c()
    for (i in 1:K2){
      c2=generate_laplace_nom(eps, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_s2[i,]-n2*p02)^2)/(n2*p02))
    t2=c(t2, t_c2)
  }
  
  x_value3=rmultinom(n=1, size=n3, prob=p13)
  x_s3=c()
  for (i in 1:K3){
    x_c3=generate_laplace_nom(eps, x=x_value3[i], size=1, n3)
    x_s3=c(x_s3, x_c3)
  }
  for (i in 2:size){
    x_value3=rmultinom(n=1, size=n3, prob=p13)
    x_c3=c()
    for (i in 1:K3){
      c3=generate_laplace_nom(eps, x=x_value3[i], size=1, n3)
      x_c3=c(x_c3, c3)
    }
    x_s3=rbind(x_s3, x_c3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_s3[i,]-n3*p03)^2)/(n3*p03))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  pow=100*length(which(t>=threshold))/size
  return(pow)
}

#For TLap
#n1, n2, n3 are the total counts for tables 1, 2 and 3, respectively
#p01,p02,p03 are the null hypothesized values for tables 1, 2 and 3, respectively
#p11,p12,p13 are the alternative hypothesized values for tables 1, 2 and 3, respectively
#x_value1, x_value2, x_value3 are the observed values for tables 1, 2 and 3, respectively
#alpha is the targeted level of significance
power_combine_lap_trunc=function(eps, n1, n2, n3, p01,p02,p03,p11,p12,p13, alpha, size){
  K1=length(p01)
  v1=(n1*p01*(1-p01)+2/eps^2)/n1/p01
  cov1=matrix(0, ncol=K1, nrow=K1)
  for (i in 1:K1){
    for (j in 1:K1){
      if (i!=j){
        cov1[i,j]=-sqrt(p01[i]*p01[j])
      }
      if (i==j){
        cov1[i,i]=v1[i]
      }
    }
  }
  
  K2=length(p02)
  v2=(n2*p02*(1-p02)+2/eps^2)/n2/p02
  cov2=matrix(0, ncol=K2, nrow=K2)
  for (i in 1:K2){
    for (j in 1:K2){
      if (i!=j){
        cov2[i,j]=-sqrt(p02[i]*p02[j])
      }
      if (i==j){
        cov2[i,i]=v2[i]
      }
    }
  }
  
  K3=length(p03)
  v3=(n3*p03*(1-p03)+2/eps^2)/n3/p03
  cov3=matrix(0, ncol=K3, nrow=K3)
  for (i in 1:K3){
    for (j in 1:K3){
      if (i!=j){
        cov3[i,j]=-sqrt(p03[i]*p03[j])
      }
      if (i==j){
        cov3[i,i]=v3[i]
      }
    }
  }
  
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=K1+K2+K3, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  x_value1=rmultinom(n=1, size=n1, prob=p11)
  x_s1=c()
  for (i in 1:K1){
    x_c1=generate_laplace_nom_trunc(eps, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, max(x_c1, 0))
  }
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p11)
    x_c1=c()
    for (i in 1:K1){
      c1=generate_laplace_nom_trunc(eps, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, max(c1, 0))
    }
    x_s1=rbind(x_s1, x_c1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_s1[i,]-n1*p01)^2)/(n1*p01))
    t1=c(t1, t_c1)
  }
  
  
  x_value2=rmultinom(n=1, size=n2, prob=p12)
  x_s2=c()
  for (i in 1:K2){
    x_c2=generate_laplace_nom_trunc(eps, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, max(x_c2, 0))
  }
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p12)
    x_c2=c()
    for (i in 1:K2){
      c2=generate_laplace_nom_trunc(eps, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_s2[i,]-n2*p02)^2)/(n2*p02))
    t2=c(t2, t_c2)
  }
  
  x_value3=rmultinom(n=1, size=n3, prob=p13)
  x_s3=c()
  for (i in 1:K3){
    x_c3=generate_laplace_nom_trunc(eps, x=x_value3[i], size=1, n3)
    x_s3=c(x_s3, max(x_c3, 0))
  }
  for (i in 2:size){
    x_value3=rmultinom(n=1, size=n3, prob=p13)
    x_c3=c()
    for (i in 1:K3){
      c3=generate_laplace_nom_trunc(eps, x=x_value3[i], size=1, n3)
      x_c3=c(x_c3, max(c3, 0))
    }
    x_s3=rbind(x_s3, x_c3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_s3[i,]-n3*p03)^2)/(n3*p03))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  pow=100*length(which(t>=threshold))/size
  return(pow)
}


#Specifically tackling the Income variable in the real data
#For Lap
#n1, n2, n3 are the total counts for tables 1, 2 and 3, respectively
#p0 is the common null hypothesized value
#merge_idx is a list of groups of indices of table cells to merge
#x_value1, x_value2, x_value3 are the observed values for tables 1, 2 and 3, respectively
#alpha is the targeted level of significance
p_value_multiple_lap_intercell_merge=function(eps, size, n1, n2, n3, p0, alpha, x_value1,x_value2, x_value3, merge_idx){
  K=length(p0)
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  v=count*2/eps^2
  variance1=(n1*p_merge*(1-p_merge)+v)/n1/p_merge
  cov1=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        cov1[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        cov1[i,i]=variance1[i]
      }
    }
  }
  variance2=(n2*p_merge*(1-p_merge)+v)/n2/p_merge
  cov2=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        cov2[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        cov2[i,i]=variance2[i]
      }
    }
  }
  variance3=(n3*p_merge*(1-p_merge)+v)/n3/p_merge
  cov3=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        cov3[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        cov3[i,i]=variance3[i]
      }
    }
  }
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:10000){
    stat=sum(lam*rchisq(n=3*length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  #Generate statistics
  x_s1=c()
  for (i in 1:K){
    x_c1=generate_laplace_nom(eps, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  x_merge1=c()
  for (i in merge_idx){
    x_merge1=c(x_merge1, sum(x_s1[i]))
  }
  
  for (i in 2:size){
    x_c1=c()
    for (i in 1:K){
      c1=generate_laplace_nom(eps, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_m1=c()
    for (i in merge_idx){
      x_m1=c(x_m1, sum(x_c1[i]))
    }
    x_merge1=rbind(x_merge1, x_m1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_merge1[i,]-n1*p_merge)^2)/(n1*p_merge))
    t1=c(t1, t_c1)
  }
  
  #From second population
  x_s2=c()
  for (i in 1:K){
    x_c2=generate_laplace_nom(eps, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  x_merge2=c()
  for (i in merge_idx){
    x_merge2=c(x_merge2, sum(x_s2[i]))
  }
  
  for (i in 2:size){
    
    x_c2=c()
    for (i in 1:K){
      c2=generate_laplace_nom(eps, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_m2=c()
    for (i in merge_idx){
      x_m2=c(x_m2, sum(x_c2[i]))
    }
    x_merge2=rbind(x_merge2, x_m2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_merge2[i,]-n2*p_merge)^2)/(n2*p_merge))
    t2=c(t2, t_c2)
  }
  
  #From third population
  x_s3=c()
  for (i in 1:K){
    x_c3=generate_laplace_nom(eps, x=x_value3[i], size=1, n3)
    x_s3=c(x_s3, x_c3)
  }
  x_merge3=c()
  for (i in merge_idx){
    x_merge3=c(x_merge3, sum(x_s3[i]))
  }
  
  for (i in 2:size){
    x_c3=c()
    for (i in 1:K){
      c3=generate_laplace_nom(eps, x=x_value3[i], size=1, n3)
      x_c3=c(x_c3, c3)
    }
    x_m3=c()
    for (i in merge_idx){
      x_m3=c(x_m3, sum(x_c3[i]))
    }
    x_merge3=rbind(x_merge3, x_m3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_merge3[i,]-n3*p_merge)^2)/(n3*p_merge))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  
  stats_derived=sort(stats_derived)
  p_v=c()
  for (i in 1:length(t)){
    p_v=c(p_v, which(stats_derived>t[i])[1]/10000)
  }
  if (is.na(mean(p_v, na.rm=T)==T)){
    return(0)
  }
  else{
    return(1-mean(p_v, na.rm=T))
  }
}

#For TLap
#n1, n2, n3 are the total counts for tables 1, 2 and 3, respectively
#p0 is the common null hypothesized value
#merge_idx is a list of groups of indices of table cells to merge
#x_value1, x_value2, x_value3 are the observed values for tables 1, 2 and 3, respectively
#alpha is the targeted level of significance
p_value_multiple_lap_intercell_merge_trunc=function(eps, size, n1, n2, n3, p0, alpha, x_value1,x_value2, x_value3, merge_idx){
  K=length(p0)
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  #p_merge=c(sum(p0[1:C]), p0[(C+1):K])
  v=count*2/eps^2
  variance1=(n1*p_merge*(1-p_merge)+v)/n1/p_merge
  cov1=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        cov1[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        cov1[i,i]=variance1[i]
      }
    }
  }
  variance2=(n2*p_merge*(1-p_merge)+v)/n2/p_merge
  cov2=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        cov2[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        cov2[i,i]=variance2[i]
      }
    }
  }
  variance3=(n3*p_merge*(1-p_merge)+v)/n3/p_merge
  cov3=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        cov3[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        cov3[i,i]=variance3[i]
      }
    }
  }
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:10000){
    stat=sum(lam*rchisq(n=3*length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  #Generate statistics
  x_s1=c()
  for (i in 1:K){
    x_c1=generate_laplace_nom_trunc(eps, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  x_merge1=c()
  for (i in merge_idx){
    x_merge1=c(x_merge1, sum(x_s1[i]))
  }
  
  for (i in 2:size){
    x_c1=c()
    for (i in 1:K){
      c1=generate_laplace_nom_trunc(eps, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_m1=c()
    for (i in merge_idx){
      x_m1=c(x_m1, sum(x_c1[i]))
    }
    x_merge1=rbind(x_merge1, x_m1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_merge1[i,]-n1*p_merge)^2)/(n1*p_merge))
    t1=c(t1, t_c1)
  }
  
  #From second population
  x_s2=c()
  for (i in 1:K){
    x_c2=generate_laplace_nom_trunc(eps, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  x_merge2=c()
  for (i in merge_idx){
    x_merge2=c(x_merge2, sum(x_s2[i]))
  }
  
  for (i in 2:size){
    x_c2=c()
    for (i in 1:K){
      c2=generate_laplace_nom_trunc(eps, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_m2=c()
    for (i in merge_idx){
      x_m2=c(x_m2, sum(x_c2[i]))
    }
    x_merge2=rbind(x_merge2, x_m2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_merge2[i,]-n2*p_merge)^2)/(n2*p_merge))
    t2=c(t2, t_c2)
  }
  
  #From third population
  x_s3=c()
  for (i in 1:K){
    x_c3=generate_laplace_nom_trunc(eps, x=x_value3[i], size=1, n3)
    x_s3=c(x_s3, x_c3)
  }
  x_merge3=c()
  for (i in merge_idx){
    x_merge3=c(x_merge3, sum(x_s3[i]))
  }
  
  for (i in 2:size){
    x_c3=c()
    for (i in 1:K){
      c3=generate_laplace_nom_trunc(eps, x=x_value3[i], size=1, n3)
      x_c3=c(x_c3, c3)
    }
    x_m3=c()
    for (i in merge_idx){
      x_m3=c(x_m3, sum(x_c3[i]))
    }
    x_merge3=rbind(x_merge3, x_m3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_merge3[i,]-n3*p_merge)^2)/(n3*p_merge))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  
  stats_derived=sort(stats_derived)
  p_v=c()
  for (i in 1:length(t)){
    p_v=c(p_v, which(stats_derived>t[i])[1]/10000)
  }
  if (is.na(mean(p_v, na.rm=T)==T)){
    return(0)
  }
  else{
    return(1-mean(p_v, na.rm=T))
  }
}



####Functions for Gaussian mechanism/Truncated Gaussian mechanism####

##To generate private data from GDP
generate_GDP=function(eps, delta, x, size, n){
  L=rnorm(n=size, mean=0, sd=sqrt(2*log(1.25/delta)-1)/eps)
  dat=round(x+L)
  return(dat)
}

##To generate private data from TGDP
generate_GDP_trunc=function(eps, delta, x, size, n){
  L=rnorm(n=size, mean=0, sd=sqrt(2*log(1.25/delta)-1)/eps)
  dat=round(x+L)
  dat[dat<0]=0
  return(dat)
}


###Goodness of fit testing###

##Using the naive Chi-squared statistics
#For GDP
#n is the total table count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples
type1_GDP_naive=function(eps, delta, n, p0, alpha, size){
  K=length(p0)
  x_value=rmultinom(n=1, size=n, prob=p0)
  x_s=c()
  for (i in 1:K){
    x_c=generate_GDP(eps, delta, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p0)
    x_c=c()
    for (i in 1:K){
      c=generate_GDP(eps, delta, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p0)^2)/(n*p0))
    t=c(t, t_c)
  }
  threshold=qchisq(1-alpha, df=K-1)  
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}

#For TGDP
#n is the total table count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples
type1_GDP_naive_trunc=function(eps, delta, n, p0, alpha, size){
  K=length(p0)
  x_value=rmultinom(n=1, size=n, prob=p0)
  x_s=c()
  for (i in 1:K){
    x_c=generate_GDP_trunc(eps, delta, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p0)
    x_c=c()
    for (i in 1:K){
      c=generate_GDP_trunc(eps, delta, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p0)^2)/(n*p0))
    t=c(t, t_c)
  }
  threshold=qchisq(1-alpha, df=K-1)  
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}

#To evaluate the empirical type I errors using the proposed procedure as in Theorem 2
#For GDP
#n is the total table count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples

type1_GDP=function(eps, delta, n, p0, alpha, size){
  K=length(p0)
  variance=(n*p0*(1-p0)+(2*log(1.25/delta)-1)/eps^2)/n/p0
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  x_value=rmultinom(n=1, size=n, prob=p0)
  x_s=c()
  for (i in 1:K){
    x_c=generate_GDP(eps, delta, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p0)
    x_c=c()
    for (i in 1:K){
      c=generate_GDP(eps, delta, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p0)^2)/(n*p0))
    t=c(t, t_c)
  }
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}

#For TGDP
#n is the total table count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples
type1_GDP_trunc=function(eps, delta, n, p0, alpha, size){
  K=length(p0)
  variance=(n*p0*(1-p0)+(2*log(1.25/delta)-1)/eps^2)/n/p0
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  x_value=rmultinom(n=1, size=n, prob=p0)
  x_s=c()
  for (i in 1:K){
    x_c=generate_GDP_trunc(eps, delta, x=x_value[i], size=1, n)
    x_s=c(x_s, max(0, x_c))
  }
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p0)
    x_c=c()
    for (i in 1:K){
      c=generate_GDP_trunc(eps, delta, x=x_value[i], size=1, n)
      x_c=c(x_c, max(0, c))
    }
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p0)^2)/(n*p0))
    t=c(t, t_c)
  }
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}


#To evaluate empirical power
#For GDP
#n is the total table count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples
#p1 is the alternative hypothesized value
power_GDP_multiple=function(n, p0, alpha, size, p1, eps, delta){
  K=length(p0)
  variance=(n*p0*(1-p0)+(2*log(1.25/delta)-1)/eps^2)/n/p0
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  x_value=rmultinom(n=1, size=n, prob=p1)
  x_s=c()
  for (i in 1:K){
    x_c=generate_GDP(eps, delta, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p1)
    x_c=c()
    for (i in 1:K){
      c=generate_GDP(eps, delta, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p0)^2)/(n*p0))
    t=c(t, t_c)
  }
  return(100*(length(which(t>=threshold)))/size) 
}

#For TGDP
#n is the total table count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples
#p1 is the alternative hypothesized value
power_GDP_multiple_trunc=function(n, p0, alpha, size, p1, eps, delta){
  K=length(p0)
  variance=(n*p0*(1-p0)+(2*log(1.25/delta)-1)/eps^2)/n/p0
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  x_value=rmultinom(n=1, size=n, prob=p1)
  x_s=c()
  for (i in 1:K){
    x_c=generate_GDP_trunc(eps, delta, x=x_value[i], size=1, n)
    x_s=c(x_s, max(0, x_c))
  }
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p1)
    x_c=c()
    for (i in 1:K){
      c=generate_GDP_trunc(eps, delta, x=x_value[i], size=1, n)
      x_c=c(x_c, max(0, c))
    }
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p0)^2)/(n*p0))
    t=c(t, t_c)
  }
  return(100*(length(which(t>=threshold)))/size) 
}


###Inter-table merging case (2 tables)
##To evaluate empirical type I errors
#For GDP
#n1, n2 are the total counts for tables 1 and 2, respectively
#p0 is the common null hypothesized value
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples
type1_merge_GDP=function(eps, n1, n2, p0, alpha, size){
  delta1=1/n1
  delta2=1/n2
  K=length(p0)
  variance=((n1+n2)*p0*(1-p0)+(2*log(1.25/delta1)-1)/eps^2+(2*log(1.25/delta2)-1)/eps^2)/(n1+n2)/p0
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  x_value1=rmultinom(n=1, size=n1, prob=p0)
  x_s1=c()
  for (i in 1:K){
    x_c1=generate_GDP(eps, delta1, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p0)
    x_c1=c()
    for (i in 1:K){
      c1=generate_GDP(eps, delta1, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_s1=rbind(x_s1, x_c1)
  }
  x_value2=rmultinom(n=1, size=n2, prob=p0)
  x_s2=c()
  for (i in 1:K){
    x_c2=generate_GDP(eps, delta2, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p0)
    x_c2=c()
    for (i in 1:K){
      c2=generate_GDP(eps, delta2, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  
  x_s=x_s1+x_s2
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-(n1+n2)*p0)^2)/((n1+n2)*p0))
    t=c(t, t_c)
  }
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}


#For TGDP
#n1, n2 are the total counts for tables 1 and 2, respectively
#p0 is the common null hypothesized value
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples
type1_merge_GDP_trunc=function(eps, n1, n2, p0, alpha, size){
  delta1=1/n1
  delta2=1/n2
  K=length(p0)
  variance=((n1+n2)*p0*(1-p0)+(2*log(1.25/delta1)-1)/eps^2+(2*log(1.25/delta2)-1)/eps^2)/(n1+n2)/p0
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  x_value1=rmultinom(n=1, size=n1, prob=p0)
  x_s1=c()
  for (i in 1:K){
    x_c1=generate_GDP_trunc(eps, delta1, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p0)
    x_c1=c()
    for (i in 1:K){
      c1=generate_GDP_trunc(eps, delta1, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_s1=rbind(x_s1, x_c1)
  }
  x_value2=rmultinom(n=1, size=n2, prob=p0)
  x_s2=c()
  for (i in 1:K){
    x_c2=generate_GDP_trunc(eps, delta2, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p0)
    x_c2=c()
    for (i in 1:K){
      c2=generate_GDP_trunc(eps, delta2, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  
  x_s=x_s1+x_s2
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-(n1+n2)*p0)^2)/((n1+n2)*p0))
    t=c(t, t_c)
  }
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}


##To evaluate empirical powers
#For GDP
#n1, n2 are the total counts for tables 1 and 2, respectively
#p0 is the common null hypothesized value
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples
#p1 is the alternative hypothesized value
pow_merge_GDP=function(eps,n1, n2, p0, alpha, size, p1){
  delta1=1/n1
  delta2=1/n2
  K=length(p0)
  variance=((n1+n2)*p0*(1-p0)+(2*log(1.25/delta1)-1)/eps^2+(2*log(1.25/delta2)-1)/eps^2)/(n1+n2)/p0
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  x_value1=rmultinom(n=1, size=n1, prob=p1)
  x_s1=c()
  for (i in 1:K){
    x_c1=generate_GDP(eps, delta1, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p1)
    x_c1=c()
    for (i in 1:K){
      c1=generate_GDP(eps, delta1, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_s1=rbind(x_s1, x_c1)
  }
  
  x_value2=rmultinom(n=1, size=n2, prob=p1)
  x_s2=c()
  for (i in 1:K){
    x_c2=generate_GDP(eps, delta2, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p1)
    x_c2=c()
    for (i in 1:K){
      c2=generate_GDP(eps, delta2, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  
  x_s=x_s1+x_s2
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-(n1+n2)*p0)^2)/((n1+n2)*p0))
    t=c(t, t_c)
  }
  pow=100*length(which(t>=threshold))/size
  return(pow)
}


#For TGDP
#n1, n2 are the total counts for tables 1 and 2, respectively
#p0 is the common null hypothesized value
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples
#p1 is the alternative hypothesized value
pow_merge_GDP_trunc=function(eps,n1, n2, p0, alpha, size, p1){
  delta1=1/n1
  delta2=1/n2
  K=length(p0)
  variance=((n1+n2)*p0*(1-p0)+(2*log(1.25/delta1)-1)/eps^2+(2*log(1.25/delta2)-1)/eps^2)/(n1+n2)/p0
  covariance=matrix(0, ncol=K, nrow=K)
  for (i in 1:K){
    for (j in 1:K){
      if (i!=j){
        covariance[i,j]=-sqrt(p0[i]*p0[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  x_value1=rmultinom(n=1, size=n1, prob=p1)
  x_s1=c()
  for (i in 1:K){
    x_c1=generate_GDP_trunc(eps, delta1, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p1)
    x_c1=c()
    for (i in 1:K){
      c1=generate_GDP_trunc(eps, delta1, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_s1=rbind(x_s1, x_c1)
  }
  
  x_value2=rmultinom(n=1, size=n2, prob=p1)
  x_s2=c()
  for (i in 1:K){
    x_c2=generate_GDP_trunc(eps, delta2, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p1)
    x_c2=c()
    for (i in 1:K){
      c2=generate_GDP_trunc(eps, delta2, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  
  x_s=x_s1+x_s2
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-(n1+n2)*p0)^2)/((n1+n2)*p0))
    t=c(t, t_c)
  }
  pow=100*length(which(t>=threshold))/size
  return(pow)
}



####Intra-table merging (assume merge first C cells)####
##To evaluate empirical type I errors

#For GDP
#n is the total table count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples
#C is index where we want to merge 1:C entries together
typeI_multiple_GDP_intercell_merge=function(eps, size, n, p0, alpha, C){
  delta=1/n
  K=length(p0)
  p_merge=c(sum(p0[1:C]), p0[(C+1):K])
  v=c(C*(2*log(1.25/delta)-1)/eps^2, rep((2*log(1.25/delta)-1)/eps^2, K-C))
  variance=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=K-C+1, nrow=K-C+1)
  for (i in 1:(K-C+1)){
    for (j in 1:(K-C+1)){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K-C+1, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  #Generate statistics
  x_value=rmultinom(n=1, size=n, prob=p0)
  K=length(p0)
  x_s=c()
  for (i in 1:K){
    x_c=generate_GDP(eps, delta, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  x_s=c(sum(x_s[1:C]), x_s[(C+1):K])
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p0)
    x_c=c()
    for (i in 1:K){
      c=generate_GDP(eps, delta, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_c=c(sum(x_c[1:C]), x_c[(C+1):K])
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}


#For TGDP
#n is the total table count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples
#C is index where we want to merge 1:C entries together
typeI_multiple_GDP_intercell_merge_trunc=function(eps, size, n, p0, alpha, C){
  delta=1/n
  K=length(p0)
  p_merge=c(sum(p0[1:C]), p0[(C+1):K])
  v=c(C*(2*log(1.25/delta)-1)/eps^2, rep((2*log(1.25/delta)-1)/eps^2, K-C))
  variance=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=K-C+1, nrow=K-C+1)
  for (i in 1:(K-C+1)){
    for (j in 1:(K-C+1)){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K-C+1, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  #Generate statistics
  x_value=rmultinom(n=1, size=n, prob=p0)
  K=length(p0)
  x_s=c()
  for (i in 1:K){
    x_c=generate_GDP_trunc(eps, delta, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  x_s=c(sum(x_s[1:C]), x_s[(C+1):K])
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p0)
    x_c=c()
    for (i in 1:K){
      c=generate_GDP_trunc(eps, delta, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_c=c(sum(x_c[1:C]), x_c[(C+1):K])
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}


###To evaluate the empirical powers
#For GDP
#n is the total table count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#p1 is the alternative hypothesized value
#size is the number of Monte Carlo samples
#C is index where we want to merge 1:C entries together
pow_multiple_GDP_intercell_merge=function(eps, size, n, p0, alpha, p1, C){
  K=length(p0)
  p_merge=c(sum(p0[1:C]), p0[(C+1):K])
  v=c(C*(2*log(1.25/delta)-1)/eps^2, rep((2*log(1.25/delta)-1)/eps^2, K-C))
  variance=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=K-C+1, nrow=K-C+1)
  for (i in 1:(K-C+1)){
    for (j in 1:(K-C+1)){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K-C+1, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  #Generate statistics
  x_value=rmultinom(n=1, size=n, prob=p1)
  K=length(p0)
  x_s=c()
  for (i in 1:K){
    x_c=generate_GDP(eps, delta, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  x_s=c(sum(x_s[1:C]), x_s[(C+1):K])
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p1)
    x_c=c()
    for (i in 1:K){
      c=generate_GDP(eps, delta, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_c=c(sum(x_c[1:C]), x_c[(C+1):K])
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  pow=100*length(which(t>=threshold))/size
  return(pow)
}


#For TGDP
#n is the total table count
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#p1 is the alternative hypothesized value
#size is the number of Monte Carlo samples
#C is index where we want to merge 1:C entries together
pow_multiple_GDP_intercell_merge_trunc=function(eps, size, n, p0, alpha, p1, C){
  K=length(p0)
  p_merge=c(sum(p0[1:C]), p0[(C+1):K])
  v=c(C*(2*log(1.25/delta)-1)/eps^2, rep((2*log(1.25/delta)-1)/eps^2, K-C))
  variance=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=K-C+1, nrow=K-C+1)
  for (i in 1:(K-C+1)){
    for (j in 1:(K-C+1)){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K-C+1, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  #Generate statistics
  x_value=rmultinom(n=1, size=n, prob=p1)
  K=length(p0)
  x_s=c()
  for (i in 1:K){
    x_c=generate_GDP_trunc(eps, delta, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  x_s=c(sum(x_s[1:C]), x_s[(C+1):K])
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p1)
    x_c=c()
    for (i in 1:K){
      c=generate_GDP_trunc(eps, delta, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_c=c(sum(x_c[1:C]), x_c[(C+1):K])
    x_s=rbind(x_s, x_c)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_s[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  pow=100*length(which(t>=threshold))/size
  return(pow)
}


####For real data application only--combine multiple test statistics###

##To evaluate empirical type I errors
#For GDP
#n1, n2, n3 are the total count for tables 1, 2 and 3, respectively
#p01, p02, p03 are the null hypothesized values for tables 1, 2 and 3, respectively
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples
type1_combine_GDP=function(eps, del1, del2, del3, n1,n2, n3, p01,p02,p03, alpha, size){
  K1=length(p01)
  v1=(n1*p01*(1-p01)+(2*log(1.25/del1)-1)/eps^2)/n1/p01
  cov1=matrix(0, ncol=K1, nrow=K1)
  for (i in 1:K1){
    for (j in 1:K1){
      if (i!=j){
        cov1[i,j]=-sqrt(p01[i]*p01[j])
      }
      if (i==j){
        cov1[i,i]=v1[i]
      }
    }
  }
  
  K2=length(p02)
  v2=(n2*p02*(1-p02)+(2*log(1.25/del2)-1)/eps^2)/n2/p02
  cov2=matrix(0, ncol=K2, nrow=K2)
  for (i in 1:K2){
    for (j in 1:K2){
      if (i!=j){
        cov2[i,j]=-sqrt(p02[i]*p02[j])
      }
      if (i==j){
        cov2[i,i]=v2[i]
      }
    }
  }
  K3=length(p03)
  v3=(n3*p03*(1-p03)+(2*log(1.25/del3)-1)/eps^2)/n3/p03
  cov3=matrix(0, ncol=K3, nrow=K3)
  for (i in 1:K3){
    for (j in 1:K3){
      if (i!=j){
        cov3[i,j]=-sqrt(p03[i]*p03[j])
      }
      if (i==j){
        cov3[i,i]=v3[i]
      }
    }
  }
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K1+K2+K3, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  
  x_value1=rmultinom(n=1, size=n1, prob=p01)
  x_s1=c()
  for (i in 1:K1){
    x_c1=generate_GDP(eps, del1, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p01)
    x_c1=c()
    for (i in 1:K1){
      c1=generate_GDP(eps, del1, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_s1=rbind(x_s1, x_c1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_s1[i,]-n1*p01)^2)/(n1*p01))
    t1=c(t1, t_c1)
  }
  
  x_value2=rmultinom(n=1, size=n2, prob=p02)
  x_s2=c()
  for (i in 1:K2){
    x_c2=generate_GDP(eps, del2, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p02)
    x_c2=c()
    for (i in 1:K2){
      c2=generate_GDP(eps, del2, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_s2[i,]-n2*p02)^2)/(n2*p02))
    t2=c(t2, t_c2)
  }
  
  x_value3=rmultinom(n=1, size=n3, prob=p03)
  x_s3=c()
  for (i in 1:K3){
    x_c3=generate_GDP(eps, del3, x=x_value3[i], size=1, n3)
    x_s3=c(x_s3, x_c3)
  }
  
  for (i in 2:size){
    x_value3=rmultinom(n=1, size=n3, prob=p03)
    x_c3=c()
    for (i in 1:K3){
      c3=generate_GDP(eps, del3, x=x_value3[i], size=1, n3)
      x_c3=c(x_c3, c3)
    }
    x_s3=rbind(x_s3, x_c3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_s3[i,]-n3*p03)^2)/(n3*p03))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}


#For TGDP
#n1, n2, n3 are the total count for tables 1, 2 and 3, respectively
#p01, p02, p03 are the null hypothesized values for tables 1, 2 and 3, respectively
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples
type1_combine_GDP_trunc=function(eps, del1, del2, del3, n1,n2, n3, p01,p02,p03, alpha, size){
  K1=length(p01)
  v1=(n1*p01*(1-p01)+(2*log(1.25/del1)-1)/eps^2)/n1/p01
  cov1=matrix(0, ncol=K1, nrow=K1)
  for (i in 1:K1){
    for (j in 1:K1){
      if (i!=j){
        cov1[i,j]=-sqrt(p01[i]*p01[j])
      }
      if (i==j){
        cov1[i,i]=v1[i]
      }
    }
  }
  
  K2=length(p02)
  v2=(n2*p02*(1-p02)+(2*log(1.25/del2)-1)/eps^2)/n2/p02
  cov2=matrix(0, ncol=K2, nrow=K2)
  for (i in 1:K2){
    for (j in 1:K2){
      if (i!=j){
        cov2[i,j]=-sqrt(p02[i]*p02[j])
      }
      if (i==j){
        cov2[i,i]=v2[i]
      }
    }
  }
  K3=length(p03)
  v3=(n3*p03*(1-p03)+(2*log(1.25/del3)-1)/eps^2)/n3/p03
  cov3=matrix(0, ncol=K3, nrow=K3)
  for (i in 1:K3){
    for (j in 1:K3){
      if (i!=j){
        cov3[i,j]=-sqrt(p03[i]*p03[j])
      }
      if (i==j){
        cov3[i,i]=v3[i]
      }
    }
  }
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K1+K2+K3, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  
  x_value1=rmultinom(n=1, size=n1, prob=p01)
  x_s1=c()
  for (i in 1:K1){
    x_c1=generate_GDP(eps, del1, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p01)
    x_c1=c()
    for (i in 1:K1){
      c1=generate_GDP_trunc(eps, del1, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, max(c1,0))
    }
    x_s1=rbind(x_s1, x_c1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_s1[i,]-n1*p01)^2)/(n1*p01))
    t1=c(t1, t_c1)
  }
  
  x_value2=rmultinom(n=1, size=n2, prob=p02)
  x_s2=c()
  for (i in 1:K2){
    x_c2=generate_GDP_trunc(eps, del2, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, max(x_c2, 0))
  }
  
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p02)
    x_c2=c()
    for (i in 1:K2){
      c2=generate_GDP_trunc(eps, del2, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, max(c2,0))
    }
    x_s2=rbind(x_s2, x_c2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_s2[i,]-n2*p02)^2)/(n2*p02))
    t2=c(t2, t_c2)
  }
  
  x_value3=rmultinom(n=1, size=n3, prob=p03)
  x_s3=c()
  for (i in 1:K3){
    x_c3=generate_GDP_trunc(eps, del3, x=x_value3[i], size=1, n3)
    x_s3=c(x_s3, max(x_c3,0))
  }
  
  for (i in 2:size){
    x_value3=rmultinom(n=1, size=n3, prob=p03)
    x_c3=c()
    for (i in 1:K3){
      c3=generate_GDP_trunc(eps, del3, x=x_value3[i], size=1, n3)
      x_c3=c(x_c3, max(c3,0))
    }
    x_s3=rbind(x_s3, x_c3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_s3[i,]-n3*p03)^2)/(n3*p03))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}

###Evaluate the average p-value
#For GDP
#n1, n2, n3 are the total count for tables 1, 2 and 3, respectively
#p01, p02, p03 are the null hypothesized values for tables 1, 2 and 3, respectively
#x_value1,x_value2,x_value3 are observed values for tables 1, 2 and 3, respectively
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples

p_val_combine_GDP=function(eps, del1, del2, del3, n1,n2, n3, p01,p02,p03, x_value1,x_value2,x_value3,alpha, size){
  K1=length(p01)
  v1=(n1*p01*(1-p01)+(2*log(1.25/del1)-1)/eps^2)/n1/p01
  cov1=matrix(0, ncol=K1, nrow=K1)
  for (i in 1:K1){
    for (j in 1:K1){
      if (i!=j){
        cov1[i,j]=-sqrt(p01[i]*p01[j])
      }
      if (i==j){
        cov1[i,i]=v1[i]
      }
    }
  }
  
  K2=length(p02)
  v2=(n2*p02*(1-p02)+(2*log(1.25/del2)-1)/eps^2)/n2/p02
  cov2=matrix(0, ncol=K2, nrow=K2)
  for (i in 1:K2){
    for (j in 1:K2){
      if (i!=j){
        cov2[i,j]=-sqrt(p02[i]*p02[j])
      }
      if (i==j){
        cov2[i,i]=v2[i]
      }
    }
  }
  K3=length(p03)
  v3=(n3*p03*(1-p03)+(2*log(1.25/del3)-1)/eps^2)/n3/p03
  cov3=matrix(0, ncol=K3, nrow=K3)
  for (i in 1:K3){
    for (j in 1:K3){
      if (i!=j){
        cov3[i,j]=-sqrt(p03[i]*p03[j])
      }
      if (i==j){
        cov3[i,i]=v3[i]
      }
    }
  }
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K1+K2+K3, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }

  x_s1=c()
  for (i in 1:K1){
    x_c1=generate_GDP(eps, del1, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  
  for (i in 2:size){

    x_c1=c()
    for (i in 1:K1){
      c1=generate_GDP(eps, del1, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_s1=rbind(x_s1, x_c1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_s1[i,]-n1*p01)^2)/(n1*p01))
    t1=c(t1, t_c1)
  }
  

  x_s2=c()
  for (i in 1:K2){
    x_c2=generate_GDP(eps, del2, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  
  for (i in 2:size){
   
    x_c2=c()
    for (i in 1:K2){
      c2=generate_GDP(eps, del2, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_s2[i,]-n2*p02)^2)/(n2*p02))
    t2=c(t2, t_c2)
  }
  

  x_s3=c()
  for (i in 1:K3){
    x_c3=generate_GDP(eps, del3, x=x_value3[i], size=1, n3)
    x_s3=c(x_s3, x_c3)
  }
  
  for (i in 2:size){
 
    x_c3=c()
    for (i in 1:K3){
      c3=generate_GDP(eps, del3, x=x_value3[i], size=1, n3)
      x_c3=c(x_c3, c3)
    }
    x_s3=rbind(x_s3, x_c3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_s3[i,]-n3*p03)^2)/(n3*p03))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  stats_derived=sort(stats_derived)
  p_v=c()
  for (i in 1:length(t)){
    p_v=c(p_v, which(stats_derived>t[i])[1]/2000)
  }
  return(1- mean(p_v, na.rm=T))
}


#For TGDP
#n1, n2, n3 are the total count for tables 1, 2 and 3, respectively
#p01, p02, p03 are the null hypothesized values for tables 1, 2 and 3, respectively
#x_value1,x_value2,x_value3 are observed values for tables 1, 2 and 3, respectively
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples
p_val_combine_GDP_trunc=function(eps, del1, del2, del3, n1,n2, n3, p01,p02,p03, x_value1,x_value2,x_value3,alpha, size){
  K1=length(p01)
  v1=(n1*p01*(1-p01)+(2*log(1.25/del1)-1)/eps^2)/n1/p01
  cov1=matrix(0, ncol=K1, nrow=K1)
  for (i in 1:K1){
    for (j in 1:K1){
      if (i!=j){
        cov1[i,j]=-sqrt(p01[i]*p01[j])
      }
      if (i==j){
        cov1[i,i]=v1[i]
      }
    }
  }
  
  K2=length(p02)
  v2=(n2*p02*(1-p02)+(2*log(1.25/del2)-1)/eps^2)/n2/p02
  cov2=matrix(0, ncol=K2, nrow=K2)
  for (i in 1:K2){
    for (j in 1:K2){
      if (i!=j){
        cov2[i,j]=-sqrt(p02[i]*p02[j])
      }
      if (i==j){
        cov2[i,i]=v2[i]
      }
    }
  }
  K3=length(p03)
  v3=(n3*p03*(1-p03)+(2*log(1.25/del3)-1)/eps^2)/n3/p03
  cov3=matrix(0, ncol=K3, nrow=K3)
  for (i in 1:K3){
    for (j in 1:K3){
      if (i!=j){
        cov3[i,j]=-sqrt(p03[i]*p03[j])
      }
      if (i==j){
        cov3[i,i]=v3[i]
      }
    }
  }
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K1+K2+K3, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }

  x_s1=c()
  for (i in 1:K1){
    x_c1=generate_GDP_trunc(eps, del1, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  
  for (i in 2:size){
 
    x_c1=c()
    for (i in 1:K1){
      c1=generate_GDP_trunc(eps, del1, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_s1=rbind(x_s1, x_c1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_s1[i,]-n1*p01)^2)/(n1*p01))
    t1=c(t1, t_c1)
  }
  

  x_s2=c()
  for (i in 1:K2){
    x_c2=generate_GDP_trunc(eps, del2, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  
  for (i in 2:size){
   
    x_c2=c()
    for (i in 1:K2){
      c2=generate_GDP_trunc(eps, del2, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_s2[i,]-n2*p02)^2)/(n2*p02))
    t2=c(t2, t_c2)
  }
  
  x_s3=c()
  for (i in 1:K3){
    x_c3=generate_GDP_trunc(eps, del3, x=x_value3[i], size=1, n3)
    x_s3=c(x_s3, x_c3)
  }
  
  for (i in 2:size){
    x_c3=c()
    for (i in 1:K3){
      c3=generate_GDP_trunc(eps, del3, x=x_value3[i], size=1, n3)
      x_c3=c(x_c3, c3)
    }
    x_s3=rbind(x_s3, x_c3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_s3[i,]-n3*p03)^2)/(n3*p03))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  stats_derived=sort(stats_derived)
  p_v=c()
  for (i in 1:length(t)){
    p_v=c(p_v, which(stats_derived>t[i])[1]/2000)
  }
  return(1- mean(p_v, na.rm=T))
}


###To evaluate empirical power
#For GDP
#n1, n2, n3 are the total count for tables 1, 2 and 3, respectively
#p01, p02, p03 are the null hypothesized values for tables 1, 2 and 3, respectively
#p11, p12, p13 are the alternative hypothesized values for tables 1, 2 and 3, respectively
#x_value1,x_value2,x_value3 are observed values for tables 1, 2 and 3, respectively
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples


pow_combine_GDP=function(eps, del1, del2, del3, n1,n2, n3, p01,p02,p03, p11, p12, p13, alpha, size){
  K1=length(p01)
  v1=(n1*p01*(1-p01)+(2*log(1.25/del1)-1)/eps^2)/n1/p01
  cov1=matrix(0, ncol=K1, nrow=K1)
  for (i in 1:K1){
    for (j in 1:K1){
      if (i!=j){
        cov1[i,j]=-sqrt(p01[i]*p01[j])
      }
      if (i==j){
        cov1[i,i]=v1[i]
      }
    }
  }
  
  K2=length(p02)
  v2=(n2*p02*(1-p02)+(2*log(1.25/del2)-1)/eps^2)/n2/p02
  cov2=matrix(0, ncol=K2, nrow=K2)
  for (i in 1:K2){
    for (j in 1:K2){
      if (i!=j){
        cov2[i,j]=-sqrt(p02[i]*p02[j])
      }
      if (i==j){
        cov2[i,i]=v2[i]
      }
    }
  }
  K3=length(p03)
  v3=(n3*p03*(1-p03)+(2*log(1.25/del3)-1)/eps^2)/n3/p03
  cov3=matrix(0, ncol=K3, nrow=K3)
  for (i in 1:K3){
    for (j in 1:K3){
      if (i!=j){
        cov3[i,j]=-sqrt(p03[i]*p03[j])
      }
      if (i==j){
        cov3[i,i]=v3[i]
      }
    }
  }
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K1+K2+K3, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  
  x_value1=rmultinom(n=1, size=n1, prob=p11)
  x_s1=c()
  for (i in 1:K1){
    x_c1=generate_GDP(eps, del1, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p11)
    x_c1=c()
    for (i in 1:K1){
      c1=generate_GDP(eps, del1, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_s1=rbind(x_s1, x_c1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_s1[i,]-n1*p01)^2)/(n1*p01))
    t1=c(t1, t_c1)
  }
  
  x_value2=rmultinom(n=1, size=n2, prob=p12)
  x_s2=c()
  for (i in 1:K2){
    x_c2=generate_GDP(eps, del2, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p12)
    x_c2=c()
    for (i in 1:K2){
      c2=generate_GDP(eps, del2, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_s2=rbind(x_s2, x_c2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_s2[i,]-n2*p02)^2)/(n2*p02))
    t2=c(t2, t_c2)
  }
  
  x_value3=rmultinom(n=1, size=n3, prob=p13)
  x_s3=c()
  for (i in 1:K3){
    x_c3=generate_GDP(eps, del3, x=x_value3[i], size=1, n3)
    x_s3=c(x_s3, x_c3)
  }
  
  for (i in 2:size){
    x_value3=rmultinom(n=1, size=n3, prob=p13)
    x_c3=c()
    for (i in 1:K3){
      c3=generate_GDP(eps, del3, x=x_value3[i], size=1, n3)
      x_c3=c(x_c3, c3)
    }
    x_s3=rbind(x_s3, x_c3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_s3[i,]-n3*p03)^2)/(n3*p03))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  pow=100*length(which(t>=threshold))/size
  return(pow)
}

#For TGDP
#n1, n2, n3 are the total count for tables 1, 2 and 3, respectively
#p01, p02, p03 are the null hypothesized values for tables 1, 2 and 3, respectively
#p11, p12, p13 are the alternative hypothesized values for tables 1, 2 and 3, respectively
#x_value1,x_value2,x_value3 are observed values for tables 1, 2 and 3, respectively
#alpha is the targeted level of significance
#size is the number of Monte Carlo samples

pow_combine_GDP_trunc=function(eps, del1, del2, del3, n1,n2, n3, p01,p02,p03, p11, p12, p13, alpha, size){
  K1=length(p01)
  v1=(n1*p01*(1-p01)+(2*log(1.25/del1)-1)/eps^2)/n1/p01
  cov1=matrix(0, ncol=K1, nrow=K1)
  for (i in 1:K1){
    for (j in 1:K1){
      if (i!=j){
        cov1[i,j]=-sqrt(p01[i]*p01[j])
      }
      if (i==j){
        cov1[i,i]=v1[i]
      }
    }
  }
  
  K2=length(p02)
  v2=(n2*p02*(1-p02)+(2*log(1.25/del2)-1)/eps^2)/n2/p02
  cov2=matrix(0, ncol=K2, nrow=K2)
  for (i in 1:K2){
    for (j in 1:K2){
      if (i!=j){
        cov2[i,j]=-sqrt(p02[i]*p02[j])
      }
      if (i==j){
        cov2[i,i]=v2[i]
      }
    }
  }
  K3=length(p03)
  v3=(n3*p03*(1-p03)+(2*log(1.25/del3)-1)/eps^2)/n3/p03
  cov3=matrix(0, ncol=K3, nrow=K3)
  for (i in 1:K3){
    for (j in 1:K3){
      if (i!=j){
        cov3[i,j]=-sqrt(p03[i]*p03[j])
      }
      if (i==j){
        cov3[i,i]=v3[i]
      }
    }
  }
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:2000){
    stat=sum(lam*rchisq(n=K1+K2+K3, df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  
  
  x_value1=rmultinom(n=1, size=n1, prob=p11)
  x_s1=c()
  for (i in 1:K1){
    x_c1=generate_GDP_trunc(eps, del1, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, max(0, x_c1))
  }
  
  for (i in 2:size){
    x_value1=rmultinom(n=1, size=n1, prob=p11)
    x_c1=c()
    for (i in 1:K1){
      c1=generate_GDP_trunc(eps, del1, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, max(0, c1))
    }
    x_s1=rbind(x_s1, x_c1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_s1[i,]-n1*p01)^2)/(n1*p01))
    t1=c(t1, t_c1)
  }
  
  x_value2=rmultinom(n=1, size=n2, prob=p12)
  x_s2=c()
  for (i in 1:K2){
    x_c2=generate_GDP_trunc(eps, del2, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, max(0, x_c2))
  }
  
  for (i in 2:size){
    x_value2=rmultinom(n=1, size=n2, prob=p12)
    x_c2=c()
    for (i in 1:K2){
      c2=generate_GDP_trunc(eps, del2, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, max(0, c2))
    }
    x_s2=rbind(x_s2, x_c2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_s2[i,]-n2*p02)^2)/(n2*p02))
    t2=c(t2, t_c2)
  }
  
  x_value3=rmultinom(n=1, size=n3, prob=p13)
  x_s3=c()
  for (i in 1:K3){
    x_c3=generate_GDP_trunc(eps, del3, x=x_value3[i], size=1, n3)
    x_s3=c(x_s3, max(0, x_c3))
  }
  
  for (i in 2:size){
    x_value3=rmultinom(n=1, size=n3, prob=p13)
    x_c3=c()
    for (i in 1:K3){
      c3=generate_GDP_trunc(eps, del3, x=x_value3[i], size=1, n3)
      x_c3=c(x_c3, max(0, c3))
    }
    x_s3=rbind(x_s3, x_c3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_s3[i,]-n3*p03)^2)/(n3*p03))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  pow=100*length(which(t>=threshold))/size
  return(pow)
}


###Used only for the income variable###
#Evaluate empirical type I errors
#For GDP
#n is the total table count 
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#merge_idx is a list of groups of indices of table cells to merge
typeI_multiple_GDP_intercell_merge_income=function(eps, size, n, p0, alpha, merge_idx){
  K=length(p0)
  delta=1/n
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  v=count*(2*log(1.25/delta)-1)/eps^2
  variance=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  #Generate statistics
  x_value=rmultinom(n=1, size=n, prob=p0)
  x_s=c()
  for (i in 1:K){
    x_c=generate_GDP(eps, delta, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  x_merge=c()
  for (i in merge_idx){
    x_merge=c(x_merge, sum(x_s[i]))
  }
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p0)
    x_c=c()
    for (i in 1:K){
      c=generate_GDP(eps, delta, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_m=c()
    for (i in merge_idx){
      x_m=c(x_m, sum(x_c[i]))
    }
    x_merge=rbind(x_merge, x_m)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_merge[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}

#For TGDP
#n is the total table count 
#p0 is the null hypothesized value
#alpha is the targeted level of significance
#merge_idx is a list of groups of indices of table cells to merge

typeI_multiple_GDP_intercell_merge_trunc_income=function(eps, size, n, p0, alpha, merge_idx){
  K=length(p0)
  delta=1/n
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  #p_merge=c(sum(p0[1:C]), p0[(C+1):K])
  v=count*(2*log(1.25/delta)-1)/eps^2
  variance=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  #Generate statistics
  x_value=rmultinom(n=1, size=n, prob=p0)
  x_s=c()
  for (i in 1:K){
    x_c=generate_GDP_trunc(eps, delta, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  x_merge=c()
  for (i in merge_idx){
    x_merge=c(x_merge, sum(x_s[i]))
  }
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p0)
    x_c=c()
    for (i in 1:K){
      c=generate_GDP_trunc(eps, delta, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_m=c()
    for (i in merge_idx){
      x_m=c(x_m, sum(x_c[i]))
    }
    x_merge=rbind(x_merge, x_m)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_merge[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  typeI=100*length(which(t>=threshold))/size
  return(typeI)
}


#Evaluate empirical power
pow_multiple_GDP_intercell_merge_income=function(eps, size, n, p0, alpha, p1, merge_idx){
  K=length(p0)
  delta=1/n
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  #p_merge=c(sum(p0[1:C]), p0[(C+1):K])
  v=count*(2*log(1.25/delta)-1)/eps^2
  variance=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  #Generate statistics
  x_value=rmultinom(n=1, size=n, prob=p1)
  x_s=c()
  for (i in 1:K){
    x_c=generate_GDP(eps, delta, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  x_merge=c()
  for (i in merge_idx){
    x_merge=c(x_merge, sum(x_s[i]))
  }
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p1)
    x_c=c()
    for (i in 1:K){
      c=generate_GDP(eps, delta, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_m=c()
    for (i in merge_idx){
      x_m=c(x_m, sum(x_c[i]))
    }
    x_merge=rbind(x_merge, x_m)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_merge[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  pow=100*length(which(t>=threshold))/size
  return(pow)
}


pow_multiple_GDP_intercell_merge_trunc_income=function(eps, size, n, p0, alpha, p1, merge_idx){
  K=length(p0)
  delta=1/n
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  #p_merge=c(sum(p0[1:C]), p0[(C+1):K])
  v=count*(2*log(1.25/delta)-1)/eps^2
  variance=(n*p_merge*(1-p_merge)+v)/n/p_merge
  covariance=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        covariance[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        covariance[i,i]=variance[i]
      }
    }
  }
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:1000){
    stat=sum(lam*rchisq(n=length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  threshold=quantile(stats_derived, probs = 1-alpha)
  #Generate statistics
  x_value=rmultinom(n=1, size=n, prob=p1)
  x_s=c()
  for (i in 1:K){
    x_c=generate_GDP_trunc(eps, delta, x=x_value[i], size=1, n)
    x_s=c(x_s, x_c)
  }
  x_merge=c()
  for (i in merge_idx){
    x_merge=c(x_merge, sum(x_s[i]))
  }
  
  for (i in 2:size){
    x_value=rmultinom(n=1, size=n, prob=p1)
    x_c=c()
    for (i in 1:K){
      c=generate_GDP_trunc(eps, delta, x=x_value[i], size=1, n)
      x_c=c(x_c, c)
    }
    x_m=c()
    for (i in merge_idx){
      x_m=c(x_m, sum(x_c[i]))
    }
    x_merge=rbind(x_merge, x_m)
  }
  t=c()
  for (i in 1:size){
    t_c=sum(((x_merge[i,]-n*p_merge)^2)/(n*p_merge))
    t=c(t, t_c)
  }
  pow=100*length(which(t>=threshold))/size
  return(pow)
}






#Evaluate average p-values

#For GDP
#n1, n2, n3 are the total counts for tables 1, 2 and 3 respectively
#p0 is the null hypothesized value
#x_value1,x_value2,x_value3 are the observed values for tables 1, 2 and 3 respectively
#alpha is the targeted level of significance
#merge_idx is a list of groups of indices of table cells to merge
p_value_multiple_GDP_intercell_merge_income=function(eps, size, n1, n2, n3, p0, alpha, x_value1,x_value2,x_value3, merge_idx){
  K=length(p0)
  delta1=1/n1
  delta2=1/n2
  delta3=1/n3
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  #p_merge=c(sum(p0[1:C]), p0[(C+1):K])
  v1=count*(2*log(1.25/delta1)-1)/eps^2
  variance1=(n1*p_merge*(1-p_merge)+v1)/n1/p_merge
  cov1=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        cov1[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        cov1[i,i]=variance1[i]
      }
    }
  }
  v2=count*(2*log(1.25/delta2)-1)/eps^2
  variance2=(n2*p_merge*(1-p_merge)+v2)/n2/p_merge
  cov2=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        cov2[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        cov2[i,i]=variance2[i]
      }
    }
  }
  v3=count*(2*log(1.25/delta3)-1)/eps^2
  variance3=(n3*p_merge*(1-p_merge)+v3)/n3/p_merge
  cov3=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        cov3[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        cov3[i,i]=variance3[i]
      }
    }
  }
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:10000){
    stat=sum(lam*rchisq(n=3*length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  #threshold=quantile(stats_derived, probs = 1-alpha)
  #Generate statistics
  #x_value1=rmultinom(n=1, size=n1, prob=p0)
  x_s1=c()
  for (i in 1:K){
    x_c1=generate_GDP(eps, delta1, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  x_merge1=c()
  for (i in merge_idx){
    x_merge1=c(x_merge1, sum(x_s1[i]))
  }
  
  for (i in 2:size){
    #x_value1=rmultinom(n=1, size=n1, prob=p0)
    x_c1=c()
    for (i in 1:K){
      c1=generate_GDP(eps, delta1, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_m1=c()
    for (i in merge_idx){
      x_m1=c(x_m1, sum(x_c1[i]))
    }
    x_merge1=rbind(x_merge1, x_m1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_merge1[i,]-n1*p_merge)^2)/(n1*p_merge))
    t1=c(t1, t_c1)
  }
  
  #Second
  #x_value2=rmultinom(n=1, size=n2, prob=p0)
  x_s2=c()
  for (i in 1:K){
    x_c2=generate_GDP(eps, delta2, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  x_merge2=c()
  for (i in merge_idx){
    x_merge2=c(x_merge2, sum(x_s2[i]))
  }
  
  for (i in 2:size){
    #x_value2=rmultinom(n=1, size=n2, prob=p0)
    x_c2=c()
    for (i in 1:K){
      c2=generate_GDP(eps, delta2, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_m2=c()
    for (i in merge_idx){
      x_m2=c(x_m2, sum(x_c2[i]))
    }
    x_merge2=rbind(x_merge2, x_m2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_merge2[i,]-n2*p_merge)^2)/(n2*p_merge))
    t2=c(t2, t_c2)
  }
  
  #Third
  #x_value3=rmultinom(n=1, size=n3, prob=p0)
  x_s3=c()
  for (i in 1:K){
    x_c3=generate_GDP(eps, delta3, x=x_value3[i], size=1, n3)
    x_s3=c(x_s3, x_c3)
  }
  x_merge3=c()
  for (i in merge_idx){
    x_merge3=c(x_merge3, sum(x_s3[i]))
  }
  
  for (i in 2:size){
    #x_value3=rmultinom(n=1, size=n3, prob=p0)
    x_c3=c()
    for (i in 1:K){
      c3=generate_GDP(eps, delta3, x=x_value3[i], size=1, n3)
      x_c3=c(x_c3, c3)
    }
    x_m3=c()
    for (i in merge_idx){
      x_m3=c(x_m3, sum(x_c3[i]))
    }
    x_merge3=rbind(x_merge3, x_m3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_merge3[i,]-n3*p_merge)^2)/(n3*p_merge))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  
  
  stats_derived=sort(stats_derived)
  p_v=c()
  for (i in 1:length(t)){
    p_v=c(p_v, which(stats_derived>t[i])[1]/10000)
  }
  if (is.na(mean(p_v, na.rm=T)==T)){
    return(0)
  }
  else{
    return(1-mean(p_v, na.rm=T))
  }
}


#For TGDP
#n1, n2, n3 are the total counts for tables 1, 2 and 3 respectively
#p0 is the null hypothesized value
#x_value1,x_value2,x_value3 are the observed values for tables 1, 2 and 3 respectively
#alpha is the targeted level of significance
#merge_idx is a list of groups of indices of table cells to merge
p_value_multiple_GDP_intercell_merge_trunc_income=function(eps, size, n1, n2, n3, p0, alpha, x_value1,x_value2,x_value3, merge_idx){
  K=length(p0)
  delta1=1/n1
  delta2=1/n2
  delta3=1/n3
  p_merge=c()
  count=c()
  for (i in merge_idx){
    p_merge=c(p_merge, sum(p0[i]))
    count=c(count,length(i))
  }
  #p_merge=c(sum(p0[1:C]), p0[(C+1):K])
  v1=count*(2*log(1.25/delta1)-1)/eps^2
  variance1=(n1*p_merge*(1-p_merge)+v1)/n1/p_merge
  cov1=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        cov1[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        cov1[i,i]=variance1[i]
      }
    }
  }
  v2=count*(2*log(1.25/delta2)-1)/eps^2
  variance2=(n2*p_merge*(1-p_merge)+v2)/n2/p_merge
  cov2=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        cov2[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        cov2[i,i]=variance2[i]
      }
    }
  }
  v3=count*(2*log(1.25/delta3)-1)/eps^2
  variance3=(n3*p_merge*(1-p_merge)+v3)/n3/p_merge
  cov3=matrix(0, ncol=length(p_merge), length(p_merge))
  for (i in 1:length(p_merge)){
    for (j in 1:length(p_merge)){
      if (i!=j){
        cov3[i,j]=-sqrt(p_merge[i]*p_merge[j])
      }
      if (i==j){
        cov3[i,i]=variance3[i]
      }
    }
  }
  covariance=as.matrix(bdiag(list(cov1, cov2, cov3)))
  e=eigen(covariance)
  lam=e$values
  stats_derived=c()
  for (j in 1:10000){
    stat=sum(lam*rchisq(n=3*length(p_merge), df=1, ncp = 0))
    stats_derived=c(stats_derived, stat)
  }
  #threshold=quantile(stats_derived, probs = 1-alpha)
  #Generate statistics
  #x_value1=rmultinom(n=1, size=n1, prob=p0)
  x_s1=c()
  for (i in 1:K){
    x_c1=generate_GDP_trunc(eps, delta1, x=x_value1[i], size=1, n1)
    x_s1=c(x_s1, x_c1)
  }
  x_merge1=c()
  for (i in merge_idx){
    x_merge1=c(x_merge1, sum(x_s1[i]))
  }
  
  for (i in 2:size){
    #x_value1=rmultinom(n=1, size=n1, prob=p0)
    x_c1=c()
    for (i in 1:K){
      c1=generate_GDP_trunc(eps, delta1, x=x_value1[i], size=1, n1)
      x_c1=c(x_c1, c1)
    }
    x_m1=c()
    for (i in merge_idx){
      x_m1=c(x_m1, sum(x_c1[i]))
    }
    x_merge1=rbind(x_merge1, x_m1)
  }
  t1=c()
  for (i in 1:size){
    t_c1=sum(((x_merge1[i,]-n1*p_merge)^2)/(n1*p_merge))
    t1=c(t1, t_c1)
  }
  
  #Second
  #x_value2=rmultinom(n=1, size=n2, prob=p0)
  x_s2=c()
  for (i in 1:K){
    x_c2=generate_GDP_trunc(eps, delta2, x=x_value2[i], size=1, n2)
    x_s2=c(x_s2, x_c2)
  }
  x_merge2=c()
  for (i in merge_idx){
    x_merge2=c(x_merge2, sum(x_s2[i]))
  }
  
  for (i in 2:size){
    #x_value2=rmultinom(n=1, size=n2, prob=p0)
    x_c2=c()
    for (i in 1:K){
      c2=generate_GDP_trunc(eps, delta2, x=x_value2[i], size=1, n2)
      x_c2=c(x_c2, c2)
    }
    x_m2=c()
    for (i in merge_idx){
      x_m2=c(x_m2, sum(x_c2[i]))
    }
    x_merge2=rbind(x_merge2, x_m2)
  }
  t2=c()
  for (i in 1:size){
    t_c2=sum(((x_merge2[i,]-n2*p_merge)^2)/(n2*p_merge))
    t2=c(t2, t_c2)
  }
  
  #Third
  #x_value3=rmultinom(n=1, size=n3, prob=p0)
  x_s3=c()
  for (i in 1:K){
    x_c3=generate_GDP_trunc(eps, delta3, x=x_value3[i], size=1, n3)
    x_s3=c(x_s3, x_c3)
  }
  x_merge3=c()
  for (i in merge_idx){
    x_merge3=c(x_merge3, sum(x_s3[i]))
  }
  
  for (i in 2:size){
    #x_value3=rmultinom(n=1, size=n3, prob=p0)
    x_c3=c()
    for (i in 1:K){
      c3=generate_GDP_trunc(eps, delta3, x=x_value3[i], size=1, n3)
      x_c3=c(x_c3, c3)
    }
    x_m3=c()
    for (i in merge_idx){
      x_m3=c(x_m3, sum(x_c3[i]))
    }
    x_merge3=rbind(x_merge3, x_m3)
  }
  t3=c()
  for (i in 1:size){
    t_c3=sum(((x_merge3[i,]-n3*p_merge)^2)/(n3*p_merge))
    t3=c(t3, t_c3)
  }
  t=t1+t2+t3
  
  
  stats_derived=sort(stats_derived)
  p_v=c()
  for (i in 1:length(t)){
    p_v=c(p_v, which(stats_derived>t[i])[1]/10000)
  }
  if (is.na(mean(p_v, na.rm=T)==T)){
    return(0)
  }
  else{
    return(1-mean(p_v, na.rm=T))
  }
}


#####MR method######
#Assume alpha=beta, find alpha values
find_alpha=function(eps, n){
  return((exp(eps/n)-1)^(-1))
}

#Generate private data using MR method
generate_mc=function(eps, n, x, size){
  alpha=find_alpha(eps,n)
  return(rbinom(n=size, size=n, prob=(x+alpha)/(n+2*alpha)))
}





































